/*
 *  ScanGear MP for Linux
 *  Copyright CANON INC. 2007
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * NOTE:
 *  - As a special exception, this program is permissible to link with the
 *    libraries released as the binary modules.
 *  - If you write modifications of your own for these programs, it is your
 *    choice whether to permit this exception to apply to your modifications.
 *    If you do not wish that, delete this exception.
*/

#ifndef	_COLORADJUST_C_
#define	_COLORADJUST_C_

#define	_CNMS_CA_GLOBALS_
#include "coloradjust.h"
#undef	_CNMS_CA_GLOBALS_

/* #define __COLOR_ADJUST_DEBUG__ */

#include "prev_main.h"
#include "cnmsstr.h"

#include "coloradjust_bc.h"
#include "coloradjust_hg.h"
#include "coloradjust_tc.h"
#include "coloradjust_fr.h"
#include "coloradjust_mc.h"

static CNMSVoid CnmsColAdjCurveValueNormalize(
						CNMSUInt32			*curve,
						CNMSInt32			max_len,
						CNMSInt32			max_value				);

static CNMSInt32 CnmsColAdjCopyHistogram(
						CNMSUInt32			masterSrc[],
						CNMSUInt32			redSrc[],
						CNMSUInt32			greenSrc[],
						CNMSUInt32			blueSrc[],
						CNMSUInt32			graySrc[],
						CNMSUInt32			masterDst[],
						CNMSUInt32			redDst[],
						CNMSUInt32			greenDst[],
						CNMSUInt32			blueDst[],
						CNMSUInt32			grayDst[]				);


/***   ColorAdjust Dialog/widget Create & Initialize   ***/
CNMSInt32 CnmsColAdjInit(
						CNMSVoid									)
{
	CNMSInt32		ret				= CNMS_ERR;
					
	if( main_window == CNMSNULL ){
		DBGMSG( "[CnmsColAdjInit] Pointer is NULL. main_window[%p]\n",main_window );
		goto EXIT;
	}		
		
	/* Create & Initialize Brightness/Contrast Dialog	*/
	if( dialog_bright_contrast == CNMSNULL ){
		dialog_bright_contrast	= create_dialog_bright_contrast();
		bc_scale_bright 		= lookup_widget( dialog_bright_contrast, "bc_scale_bright" );
		bc_spinbutton_bright 	= lookup_widget( dialog_bright_contrast, "bc_spinbutton_bright" );
		bc_scale_contrast 		= lookup_widget( dialog_bright_contrast, "bc_scale_contrast" );
		bc_spinbutton_contrast 	= lookup_widget( dialog_bright_contrast, "bc_spinbutton_contrast" );
		bc_combo_box 			= lookup_widget( dialog_bright_contrast, "bc_combobox" );
		bc_drawarea				= lookup_widget( dialog_bright_contrast, "bc_drawingarea" );
		CnmsBCInit();
		
		bc_pos.x = CNMS_ERR;
		bc_pos.y = CNMS_ERR;
	}

	/* Create & Initialize Histogram Dialog	*/
	if( dialog_histogram == CNMSNULL ){
		dialog_histogram 		= create_dialog_histogram();
		hg_combo_box	 		= lookup_widget( dialog_histogram, "hg_combobox" );
		hg_drawarea	 			= lookup_widget( dialog_histogram, "hg_drawingarea" );
		hg_entry_shadow	 		= lookup_widget( dialog_histogram, "hg_spinbutton_shadow" );
		hg_entry_mid	 		= lookup_widget( dialog_histogram, "hg_spinbutton_mid" );
		hg_entry_highlight		= lookup_widget( dialog_histogram, "hg_spinbutton_highlight" );
		hg_scalebar_drawarea	= lookup_widget( dialog_histogram, "hg_drawingarea_scalebar" );
		hg_label_str_1	 		= lookup_widget( dialog_histogram, "hg_label_str_1" );
		hg_label_str_2	 		= lookup_widget( dialog_histogram, "hg_label_str_2" );
		hg_label_str_3	 		= lookup_widget( dialog_histogram, "hg_label_str_3" );
		hg_label_value_1 		= lookup_widget( dialog_histogram, "hg_label_value_1" );
		hg_label_value_2 		= lookup_widget( dialog_histogram, "hg_label_value_2" );
		hg_label_value_3 		= lookup_widget( dialog_histogram, "hg_label_value_3" );
		hg_togglebutton_gbal	= lookup_widget( dialog_histogram, "hg_togglebutton_graybalance" );
		CnmsHGInit( TRUE );
		
		hg_pos.x = CNMS_ERR;
		hg_pos.y = CNMS_ERR;

	}

	/* Create & Initialize ToneCurve Dialog	*/
	if( dialog_tonecurve == CNMSNULL ){
		dialog_tonecurve		= create_dialog_tonecurve();
		tc_combobox_channel		= lookup_widget( dialog_tonecurve, "tc_combobox" );
		tc_drawarea				= lookup_widget( dialog_tonecurve, "tc_drawingarea" );
		tc_combobox_tone_select	= lookup_widget( dialog_tonecurve, "tc_combobox_tc_select" );
		CnmsTCInit();

		tc_pos.x = CNMS_ERR;
		tc_pos.y = CNMS_ERR;
	}

	/* Create & Initialize FinalReview Dialog	*/
	if( dialog_final_review == CNMSNULL ){
		dialog_final_review	= create_dialog_final_review();
		fr_combo_box	 	= lookup_widget( dialog_final_review, "fr_combobox" );
		fr_drawarea	 		= lookup_widget( dialog_final_review, "fr_drawingarea" );
		fr_label_str_a1	 	= lookup_widget( dialog_final_review, "fr_label_str_a_1" );
		fr_label_str_b1	 	= lookup_widget( dialog_final_review, "fr_label_str_b_1" );
		fr_label_str_a2	 	= lookup_widget( dialog_final_review, "fr_label_str_a_2" );
		fr_label_str_b2	 	= lookup_widget( dialog_final_review, "fr_label_str_b_2" );
		fr_label_str_a3	 	= lookup_widget( dialog_final_review, "fr_label_str_a_3" );
		fr_label_str_b3	 	= lookup_widget( dialog_final_review, "fr_label_str_b_3" );
		fr_label_value_a1 	= lookup_widget( dialog_final_review, "fr_label_value_a_1" );
		fr_label_value_b1 	= lookup_widget( dialog_final_review, "fr_label_value_b_1" );
		fr_label_value_a2 	= lookup_widget( dialog_final_review, "fr_label_value_a_2" );
		fr_label_value_b2 	= lookup_widget( dialog_final_review, "fr_label_value_b_2" );
		fr_label_value_a3 	= lookup_widget( dialog_final_review, "fr_label_value_a_3" );
		fr_label_value_b3 	= lookup_widget( dialog_final_review, "fr_label_value_b_3" );

		fr_pos.x = CNMS_ERR;
		fr_pos.y = CNMS_ERR;
		CnmsFRReset( TRUE );
	}

	/* Create & Initialize Monochrome Dialog	*/
	if( dialog_monochrome == CNMSNULL ){
		dialog_monochrome 	= create_dialog_monochrome();
		mc_drawarea	 		= lookup_widget( dialog_monochrome, "mc_drawingarea" );
		mc_spinbutton	 	= lookup_widget( dialog_monochrome, "mc_spinbutton" );
		mc_scale	 		= lookup_widget( dialog_monochrome, "mc_scale" );
		CnmsMCInit( TRUE );
		
		mc_pos.x = CNMS_ERR;
		mc_pos.y = CNMS_ERR;
	}

	ret = CNMS_NO_ERR;
EXIT:

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjInit ]\n" );
#endif

	return ret;
}	


/***   Drawing Graph Base ( Frame and Line )   ***/
CNMSInt32 CnmsColAdjDrawAreaBaseDraw(
						GtkWidget			*widget,
						GdkPixmap			*pixmap,
						GdkGC				*gc						)
{
	CNMSInt32	ret		= CNMS_ERR,
				Left	= CNMS_CA_GRAPH_FRAME_LEFT,
				Top		= CNMS_CA_GRAPH_FRAME_TOP,
				Right	= CNMS_CA_GRAPH_FRAME_RIGHT,
				Bottom	= CNMS_CA_GRAPH_FRAME_BOTTOM;

	if( ( widget == CNMSNULL ) || ( pixmap == CNMSNULL ) || ( gc == CNMSNULL ) ){
		DBGMSG( "[CnmsColAdjDrawAreaBaseDraw] Arg Pointer is NULL. widget[%p], pixmap[%p], gc[%p]\n",widget,pixmap,gc );
		goto EXIT;
	}		

	/* Drawing Back ground		*/
	CnmsColAdjGdkGCSetColor( gc, CNMS_CA_GC_BACKGROUND_R, CNMS_CA_GC_BACKGROUND_G, CNMS_CA_GC_BACKGROUND_B );
	gdk_draw_rectangle( pixmap, gc, TRUE, 0, 0, widget->allocation.width, widget->allocation.height );

	/* Drawing Frame outside	*/
	CnmsColAdjGdkGCSetColor( gc, CNMS_CA_GC_FRAME_R, CNMS_CA_GC_FRAME_G, CNMS_CA_GC_FRAME_B );
	gdk_draw_rectangle( pixmap, gc, FALSE, Left, Top, Right - Left, Bottom - Top );
								
	/* Drawing Frame inside	*/
	gdk_draw_line( pixmap, gc, ( ( Right - Left ) * 1 / 4 ) + Left, Top, ( ( Right - Left ) * 1 / 4 ) + Left, Bottom ); 	/* Vertical line 1st from leftside	*/
	gdk_draw_line( pixmap, gc, ( ( Right - Left ) * 2 / 4 ) + Left, Top, ( ( Right - Left ) * 2 / 4 ) + Left, Bottom ); 	/* Vertical line 2nd from leftside	*/
	gdk_draw_line( pixmap, gc, ( ( Right - Left ) * 3 / 4 ) + Left, Top, ( ( Right - Left ) * 3 / 4 ) + Left, Bottom ); 	/* Vertical line 3rd from leftside	*/
	gdk_draw_line( pixmap, gc,   Left,  ( ( Bottom - Top ) * 1 / 4 ) + Top, Right, ( ( Bottom - Top ) * 1 / 4 ) +Top ); 	/* Horizontal line 1st from topside	*/
	gdk_draw_line( pixmap, gc, 	 Left,  ( ( Bottom - Top ) * 2 / 4 ) + Top, Right, ( ( Bottom - Top ) * 2 / 4 ) + Top ); 	/* Horizontal line 2nd from topside	*/
	gdk_draw_line( pixmap, gc,	 Left,  ( ( Bottom - Top ) * 3 / 4 ) + Top, Right, ( ( Bottom - Top ) * 3 / 4 ) + Top ); 	/* Horizontal line 3rd from topside	*/

	ret = CNMS_NO_ERR;
EXIT:

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjDrawAreaBaseDraw ]\n" );
#endif

	return ret;
}


/***   Drawing ToneCurve on Graph without Graph Base   ***/
CNMSInt32 CnmsColAdjDrawAreaCurveDraw(
						GtkWidget			*widget,
						GdkPixmap			*pixmap,
						GdkGC				*gc,
						CNMSInt32			curve[],
						CNMSInt32			color					)
{
	CNMSInt32	i		= 0,
				ret		= CNMS_ERR,
				VecLen	= CNMS_CA_VECTOR_LEN,
				Left	= CNMS_CA_GRAPH_FRAME_LEFT,
				Top		= CNMS_CA_GRAPH_FRAME_TOP,
				Right	= CNMS_CA_GRAPH_FRAME_RIGHT,
				Bottom	= CNMS_CA_GRAPH_FRAME_BOTTOM;
	
	if( ( widget == CNMSNULL ) || ( pixmap == CNMSNULL ) || ( gc == CNMSNULL ) ){
		DBGMSG( "[CnmsColAdjDrawAreaCurveDraw] Arg Pointer is NULL. widget[%p], pixmap[%p], gc[%p]\n",widget,pixmap,gc );
		goto EXIT;
	}		

	/* Set line color */
	switch( color )
	{
		case CNMS_CA_COLOR_BLACK:	CnmsColAdjGdkGCSetColor( gc, CNMS_CA_GC_VALUE_ZERO, CNMS_CA_GC_VALUE_ZERO, CNMS_CA_GC_VALUE_ZERO );	break;		/* Draw black line	*/
		case CNMS_CA_COLOR_RED:		CnmsColAdjGdkGCSetColor( gc,  CNMS_CA_GC_VALUE_MAX, CNMS_CA_GC_VALUE_ZERO, CNMS_CA_GC_VALUE_ZERO );	break;		/* Draw   red line	*/
		case CNMS_CA_COLOR_GREEN:	CnmsColAdjGdkGCSetColor( gc, CNMS_CA_GC_VALUE_ZERO,  CNMS_CA_GC_VALUE_MAX, CNMS_CA_GC_VALUE_ZERO );	break;		/* Draw green line	*/
		case CNMS_CA_COLOR_BLUE:	CnmsColAdjGdkGCSetColor( gc, CNMS_CA_GC_VALUE_ZERO, CNMS_CA_GC_VALUE_ZERO,  CNMS_CA_GC_VALUE_MAX );	break;		/* Draw  blue line	*/
		case CNMS_CA_COLOR_GRAY:	CnmsColAdjGdkGCSetColor( gc, CNMS_CA_GC_VALUE_GRAY, CNMS_CA_GC_VALUE_GRAY, CNMS_CA_GC_VALUE_GRAY );	break;		/* Draw  gray line	*/
		default:					DBGMSG("[CnmsColAdjDrawAreaCurveDraw] Arg color is invalid. color[%d]\n",color);			goto	EXIT;
	}
	
	/* Draw curve line */
	for( i = 1 ; i < VecLen ; i++ )
	{
		gdk_draw_line( pixmap, gc,
						( CNMSInt32 )( ( ( CNMSDec32 )( i - 1 ) / ( VecLen - 1 ) * ( Right - Left - 1 ) ) + 0.5 ) + ( Left + 1 ),					/* Start X */
						abs( ( CNMSInt32 )( ( ( CNMSDec32 )curve[ i - 1 ] * ( Bottom - Top - 1 ) / ( VecLen - 1 ) ) + 0.5 ) - ( Bottom - 1 ) ),		/* Start Y */
						( CNMSInt32 )( ( ( CNMSDec32 )( i     ) / ( VecLen - 1 ) * ( Right - Left - 1 ) ) + 0.5 ) + ( Left + 1 ),					/* End   X */
						abs( ( CNMSInt32 )( ( ( CNMSDec32 )curve[ i     ] * ( Bottom - Top - 1 ) / ( VecLen - 1 ) ) + 0.5 ) - ( Bottom - 1 ) ) );	/* End   Y */
	}

	/* Drawing Frame outside	*/
	CnmsColAdjGdkGCSetColor( gc, CNMS_CA_GC_FRAME_R, CNMS_CA_GC_FRAME_G, CNMS_CA_GC_FRAME_B );
	gdk_draw_rectangle( pixmap, gc, FALSE, Left, Top, Right - Left, Bottom - Top );
								
	ret = CNMS_NO_ERR;
EXIT:

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjDrawAreaCurveDraw ]\n" );
#endif

	return ret;
}


/***   Drawing Histogram on Graph without Graph Base   ***/
CNMSInt32 CnmsColAdjDrawAreaHistogramDraw(
						GtkWidget			*widget,
						GdkPixmap			*pixmap,
						GdkGC				*gc,
						CNMSUInt32			histogram[],
						CNMSInt32			color					)
{
	CNMSInt32	i		= 0,
				ret		= CNMS_ERR,
				VecLen	= CNMS_CA_VECTOR_LEN,
				Left	= CNMS_CA_GRAPH_FRAME_LEFT,
				Top		= CNMS_CA_GRAPH_FRAME_TOP,
				Right	= CNMS_CA_GRAPH_FRAME_RIGHT,
				Bottom	= CNMS_CA_GRAPH_FRAME_BOTTOM;

	if( ( widget == CNMSNULL ) || ( pixmap == CNMSNULL ) || ( gc == CNMSNULL ) ){
		DBGMSG( "[CnmsColAdjDrawAreaCurveDraw] Arg Pointer is NULL. widget[%p], pixmap[%p], gc[%p]\n",widget,pixmap,gc );
		goto EXIT;
	}		

	/* Set line color */
	switch( color )
	{
		case CNMS_CA_COLOR_BLACK:	CnmsColAdjGdkGCSetColor( gc, CNMS_CA_GC_VALUE_ZERO, CNMS_CA_GC_VALUE_ZERO, CNMS_CA_GC_VALUE_ZERO );	break;		/* Draw black line	*/
		case CNMS_CA_COLOR_RED:		CnmsColAdjGdkGCSetColor( gc,  CNMS_CA_GC_VALUE_MAX, CNMS_CA_GC_VALUE_ZERO, CNMS_CA_GC_VALUE_ZERO );	break;		/* Draw   red line	*/
		case CNMS_CA_COLOR_GREEN:	CnmsColAdjGdkGCSetColor( gc, CNMS_CA_GC_VALUE_ZERO,  CNMS_CA_GC_VALUE_MAX, CNMS_CA_GC_VALUE_ZERO );	break;		/* Draw green line	*/
		case CNMS_CA_COLOR_BLUE:	CnmsColAdjGdkGCSetColor( gc, CNMS_CA_GC_VALUE_ZERO, CNMS_CA_GC_VALUE_ZERO,  CNMS_CA_GC_VALUE_MAX );	break;		/* Draw  blue line	*/
		case CNMS_CA_COLOR_GRAY:	CnmsColAdjGdkGCSetColor( gc, CNMS_CA_GC_VALUE_GRAY, CNMS_CA_GC_VALUE_GRAY, CNMS_CA_GC_VALUE_GRAY );	break;		/* Draw  gray line	*/
		default:					DBGMSG("[CnmsColAdjDrawAreaHistogramDraw] Arg color is invalid. color[%d]\n",color);		goto	EXIT;
	}

	/* Draw histogram line */
	for( i = 0 ; i < VecLen ; i++ ){
		if( histogram[i] != 0 )
		{
			CnmsColAdjCurveValueNormalize( histogram, VecLen, VecLen-1 );

			for( i = 0 ; i < VecLen ; i++ )
			{
				gdk_draw_line( pixmap, gc,
								( CNMSInt32 )( ( ( CNMSDec32 )i / ( VecLen - 1 ) * ( Right - Left - 1 ) ) - 0.5 ) + ( Left + 1 ),						/* Start X */
								( Bottom - 1 ),																											/* Start Y */
								( CNMSInt32 )( ( ( CNMSDec32 )i / ( VecLen - 1 ) * ( Right - Left - 1 ) ) - 0.5 ) + ( Left + 1 ),						/* End   X */
								abs( ( CNMSInt32 )( ( ( CNMSDec32 )histogram[i] * ( Bottom - Top - 1 ) / ( VecLen - 1 ) ) - 0.5 ) - ( Bottom - 1 ) ) );	/* End   Y */
			}
			break;
		}
	}

	ret = CNMS_NO_ERR;
EXIT:

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjDrawAreaHistogramDraw ]\n" );
#endif

	return ret;
}


/***   ColorMode Changed ( & Move Page to Advanced )   ***/
CNMSInt32 CnmsColAdjColorModeChanged(
						CNMSVoid									)
{
	GtkWidget			*notebook		= CNMSNULL;
	static CNMSInt32 	colorBefore		= CNMS_CA_COLORGROUP_COLOR;		/* ColorGroup Before ColorMode Changed	*/
	CNMSInt32			colorAfter		= CNMS_CA_COLORGROUP_ERROR,		/* ColorGroup Current ColorMode Changed	*/
						ret				= CNMS_ERR;

	if( ( main_window == CNMSNULL ) || ( ( notebook = lookup_widget( main_window, "main_notebook" ) ) == CNMSNULL ) ){
		DBGMSG( "[CnmsColAdjColorModeChanged] Pointer is NULL. main_window[%p], notebook[%p]\n",main_window,notebook );
		goto EXIT;
	}		
	if( ( colorAfter = CnmsColAdjGetColorGroupNum() ) == CNMS_CA_COLORGROUP_ERROR ){
		DBGMSG( "[CnmsColAdjColorModeChanged] colorAfter value [%d] is invalid.\n",colorAfter );
		goto EXIT;
	}		
	if( ( dialog_bright_contrast == CNMSNULL ) || ( dialog_histogram == CNMSNULL ) || ( dialog_tonecurve == CNMSNULL ) ||
																	( dialog_final_review == CNMSNULL ) || ( dialog_monochrome == CNMSNULL ) )
	{
		DBGMSG( "[CnmsColAdjColorModeChanged] Dialog Pointer is NULL.\n" );
		goto EXIT;
	}		
		
	/* Case Changed between Same ColorGroup, No Reset Value and Disp	*/
	if( colorBefore == colorAfter ){
#ifdef	__COLOR_ADJUST_DEBUG__
		DBGMSG( "[CnmsColAdjColorModeChanged] colorBefore = colorAfter. colorBefore[%d], colorAfter[%d]\n",colorBefore,colorAfter );
#endif
		ret = CNMS_NO_ERR;
		goto EXIT;
	}

	/* Advanced mode only */
	if( gtk_notebook_get_current_page( GTK_NOTEBOOK( notebook ) ) == 1 )
	{
		/* Show/UnShow judge for Dialog */
		if( bc_pos.show == TRUE ){
			CnmsBCDialogShow( colorAfter, CNMS_CA_DIALOG_KEEP );
			bc_pos.show = TRUE;
		}
		else{
			CnmsBCDialogShow( colorAfter, CNMS_CA_DIALOG_KEEP );
		}

		if( hg_pos.show == TRUE ){
			CnmsHGDialogShow( colorAfter, CNMS_CA_DIALOG_KEEP );
			hg_pos.show = TRUE;
		}
		else{
			CnmsHGDialogShow( colorAfter, CNMS_CA_DIALOG_KEEP );
		}

		if( tc_pos.show == TRUE ){
			CnmsTCDialogShow( colorAfter, CNMS_CA_DIALOG_KEEP );
			tc_pos.show = TRUE;
		}
		else{
			CnmsTCDialogShow( colorAfter, CNMS_CA_DIALOG_KEEP );
		}

		if( fr_pos.show == TRUE ){
			CnmsFRDialogShow( colorAfter, CNMS_CA_DIALOG_KEEP );
			fr_pos.show = TRUE;
		}
		else{
			CnmsFRDialogShow( colorAfter, CNMS_CA_DIALOG_KEEP );
		}

		if( mc_pos.show == TRUE ){
			CnmsMCDialogShow( colorAfter, CNMS_CA_DIALOG_KEEP );
			mc_pos.show = TRUE;
		}
		else{
			CnmsMCDialogShow( colorAfter, CNMS_CA_DIALOG_KEEP );
		}

		/* If Chaned between differnce Group, Data Reset */
		CnmsColAdjAllDataReset();
				
		/* If Need Show but UnShow, Show Dialog */
		switch( colorAfter )
		{
			case	CNMS_CA_COLORGROUP_COLOR:
			case	CNMS_CA_COLORGROUP_GRAY:
						if( ( bc_pos.show == TRUE ) && ( GTK_WIDGET_VISIBLE( dialog_bright_contrast ) == 0 ) ){
							CnmsBCDialogShow( colorAfter, CNMS_CA_DIALOG_NON_KEEP );
						}
						if( ( hg_pos.show == TRUE ) && ( GTK_WIDGET_VISIBLE( dialog_histogram ) == 0 ) ){
							CnmsHGDialogShow( colorAfter, CNMS_CA_DIALOG_NON_KEEP );
						}
						if( ( tc_pos.show == TRUE ) && ( GTK_WIDGET_VISIBLE( dialog_tonecurve ) == 0 ) ){
							CnmsTCDialogShow( colorAfter, CNMS_CA_DIALOG_NON_KEEP );
						}
						if( ( fr_pos.show == TRUE ) && ( GTK_WIDGET_VISIBLE( dialog_final_review ) == 0 ) ){
							CnmsFRDialogShow( colorAfter, CNMS_CA_DIALOG_NON_KEEP );
						}
						break;
		
			case	CNMS_CA_COLORGROUP_MONO:
						if( ( mc_pos.show == TRUE ) && ( GTK_WIDGET_VISIBLE( dialog_monochrome ) == 0 ) ){
							CnmsMCDialogShow( colorAfter, CNMS_CA_DIALOG_NON_KEEP );
						}
						break;
		
			default:
						DBGMSG( "[CnmsColAdjColorModeChanged] colorAfter value [%d] is invalid.\n",colorAfter );
						goto EXIT;
		}
	}
	else{
		CnmsColAdjAllDataReset();
	}

	/* Keep current color group */
	colorBefore = colorAfter;
	
	ret = CNMS_NO_ERR;
EXIT:

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjColorModeChanged ]\n" );
#endif

	return ret;
}


/***   Dialog Button on Main Window Clicked   ***/
CNMSVoid CnmsColAdjButtonClicked(
						CNMSInt32			ButtonNum				)
{
	CNMSInt32		colorGroup	= CNMS_CA_COLORGROUP_ERROR;
	
	if( ( colorGroup = CnmsColAdjGetColorGroupNum() ) != CNMS_CA_COLORGROUP_ERROR )
	{
		switch( ButtonNum ){
			case	CNMS_CA_BUTTON_BC:		CnmsBCDialogShow( colorGroup, CNMS_CA_DIALOG_NON_KEEP );	break;
			case	CNMS_CA_BUTTON_HG:		CnmsHGDialogShow( colorGroup, CNMS_CA_DIALOG_NON_KEEP );	break;
			case	CNMS_CA_BUTTON_TC:		CnmsTCDialogShow( colorGroup, CNMS_CA_DIALOG_NON_KEEP );	break;
			case	CNMS_CA_BUTTON_FR:		CnmsFRDialogShow( colorGroup, CNMS_CA_DIALOG_NON_KEEP );	break;
			case	CNMS_CA_BUTTON_MC:		CnmsMCDialogShow( colorGroup, CNMS_CA_DIALOG_NON_KEEP );	break;
			default:						DBGMSG( "[CnmsColAdjButtonClicked] ButtonNum value [%d] is invalid.\n",ButtonNum );
											break;
		}
	}

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjButtonClicked ]\n" );
#endif

	return;
}


/***   All Dialog Parameter Reset only ( dialog No hide )   ***/
CNMSVoid CnmsColAdjAllDataReset(
						CNMSVoid									)
{
	if( dialog_bright_contrast != CNMSNULL ){
		CnmsBCReset();
	}
	if( dialog_histogram != CNMSNULL ){
		CnmsHGReset( TRUE, FALSE );
	}
	if( dialog_tonecurve != CNMSNULL ){
		CnmsTCReset();
	}
	if( dialog_final_review != CNMSNULL ){
		CnmsFRReset( FALSE );
	}
	if( dialog_monochrome != CNMSNULL ){
		CnmsMCReset( FALSE );
	}

	CnmsColAdjPreviewImageSet();
	
#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjAllDataReset ]\n" );
#endif

	return;
}


/***   All Dialog Parameter Reset with Dialog Hide   ***/
CNMSVoid CnmsColAdjResetComplete(
						CNMSVoid									)
{
	if( dialog_bright_contrast != CNMSNULL ){
		if( GTK_WIDGET_VISIBLE( dialog_bright_contrast ) != 0 ){
			CnmsColAdjDialogHide( dialog_bright_contrast, &bc_pos );
		}
	}
	if( dialog_histogram != CNMSNULL ){
		if( GTK_WIDGET_VISIBLE( dialog_histogram ) != 0 ){
			CnmsColAdjDialogHide( dialog_histogram, &hg_pos );
		}
	}
	if( dialog_tonecurve != CNMSNULL ){
		if( GTK_WIDGET_VISIBLE( dialog_tonecurve ) != 0 ){
			CnmsColAdjDialogHide( dialog_tonecurve, &tc_pos );
		}
	}
	if( dialog_final_review != CNMSNULL ){
		if( GTK_WIDGET_VISIBLE( dialog_final_review ) != 0 ){
			CnmsColAdjDialogHide( dialog_final_review, &fr_pos );
		}
	}
	if( dialog_monochrome != CNMSNULL ){
		if( GTK_WIDGET_VISIBLE( dialog_monochrome ) != 0 ){
			CnmsColAdjDialogHide( dialog_monochrome, &mc_pos );
		}
	}

	CnmsColAdjAllDataReset();
	
#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjResetComplete ]\n" );
#endif

	return;
}


/***   Dialog Show with Keep Position   ***/
CNMSInt32 CnmsColAdjDialogShow(
						GtkWidget			*dialog,
						CNMSDialogPos		*pos					)
{
	CNMSInt32	ret		= CNMS_ERR;

	if( ( dialog == CNMSNULL ) || ( pos == CNMSNULL ) ){
		DBGMSG( "[CnmsColAdjDialogShow] Arg Pointer is NULL. dialog[%p], pos[%p]\n",dialog,pos );
		goto EXIT;
	}		

	gtk_window_move( GTK_WINDOW( dialog ), pos->x, pos->y );
	gtk_widget_show( dialog );
	pos->show = TRUE;

	ret = CNMS_NO_ERR;
EXIT:

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjDialogShow ]\n" );
#endif

	return ret;
}

/***   Dialog Hide with Keep Position   ***/
CNMSInt32 CnmsColAdjDialogHide(
						GtkWidget			*dialog,
						CNMSDialogPos		*pos					)
{
	CNMSInt32	ret		= CNMS_ERR;

	if( ( dialog == CNMSNULL ) || ( pos == CNMSNULL ) ){
		DBGMSG( "[CnmsColAdjDialogHide] Arg Pointer is NULL. dialog[%p], pos[%p]\n",dialog,pos );
		goto EXIT;
	}		

	gtk_window_get_position( GTK_WINDOW( dialog ), &(pos->x), &(pos->y) );
	gtk_widget_hide( dialog );
	pos->show = FALSE;
	W1_HideAndShowAllButton();

	ret = CNMS_NO_ERR;
EXIT:

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjDialogHide ]\n" );
#endif

	return ret;
}


/***   Histogram Value Set   ***/
CNMSInt32 CnmsColAdjSetHistogram(
						CNMSUInt32			type,
						CNMSUInt32			masterSrc[],
						CNMSUInt32			redSrc[],
						CNMSUInt32			greenSrc[],
						CNMSUInt32			blueSrc[],
						CNMSUInt32			graySrc[]				)
{
	CNMSInt32	i		= 0,
				ret		= CNMS_ERR;

	if( masterSrc == CNMSNULL ){
		DBGMSG( "[CnmsColAdjSetHistogram] Arg Pointer is NULL. masterSrc[%p]\n",masterSrc );
		goto EXIT;
	}		

	switch( type ){
		case CNMS_CA_BUTTON_HG:
					CnmsColAdjCopyHistogram( masterSrc, redSrc, greenSrc, blueSrc, graySrc, hg_master.hist, hg_red.hist, hg_green.hist, hg_blue.hist, hg_gray.hist );
					break;
					
		case CNMS_CA_BUTTON_FR:
					CnmsColAdjCopyHistogram( masterSrc, redSrc, greenSrc, blueSrc, graySrc, fr_master.hist, fr_red.hist, fr_green.hist, fr_blue.hist, fr_gray.hist );
					break;
					
		case CNMS_CA_BUTTON_MC:
					CnmsColAdjCopyHistogram( masterSrc, CNMSNULL, CNMSNULL, CNMSNULL, CNMSNULL, mc_mono.hist, CNMSNULL, CNMSNULL, CNMSNULL, CNMSNULL	);
					break;
					
		default:
			DBGMSG( "[CnmsColAdjSetHistogram] Arg value is invalid. type[%d]\n",type );
			goto EXIT;
	}		
		
	ret = CNMS_NO_ERR;
EXIT:

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjSetHistogram ]\n" );
#endif

	return ret;
}


/***   Histogram Value Set   ***/
static CNMSInt32 CnmsColAdjCopyHistogram(
						CNMSUInt32			masterSrc[],
						CNMSUInt32			redSrc[],
						CNMSUInt32			greenSrc[],
						CNMSUInt32			blueSrc[],
						CNMSUInt32			graySrc[],
						CNMSUInt32			masterDst[],
						CNMSUInt32			redDst[],
						CNMSUInt32			greenDst[],
						CNMSUInt32			blueDst[],
						CNMSUInt32			grayDst[]				)
{
	CNMSInt32	i		= 0,
				ret		= CNMS_ERR;

	if( ( masterSrc == CNMSNULL ) || ( masterDst == CNMSNULL ) ){
		DBGMSG( "[CnmsColAdjSetHistogram] Arg Pointer is NULL. masterSrc[%p], masterDst[%p]\n",masterSrc,masterDst );
		goto EXIT;
	}		
		
	/* Master/Monochrome Initialize	with zero	*/
	for( i = 0 ; i < 256 ; i++ ){
		masterDst[i] = 0;
	}

	/* Other Initialize	with zero	*/
	if( ( redDst != CNMSNULL ) && ( greenDst != CNMSNULL ) && ( blueDst != CNMSNULL ) && ( grayDst != CNMSNULL ) ){
		for( i = 0 ; i < 256 ; i++ ){
			redDst[i] = greenDst[i] = blueDst[i] = grayDst[i] = 0;
		}
	}
	
	/* Master/Monochrome Value Copy Src to Dst	*/
	for( i = 0 ; i < 256 ; i++ ){
		masterDst[i] = masterSrc[i];
	}

	/* Other Value Copy Src to Dst	*/
	if( ( redDst != CNMSNULL ) && ( greenDst != CNMSNULL ) && ( blueDst != CNMSNULL ) && ( grayDst != CNMSNULL ) &&
		( redSrc != CNMSNULL ) && ( greenSrc != CNMSNULL ) && ( blueSrc != CNMSNULL ) && ( graySrc != CNMSNULL ) )
	{
		for( i = 0 ; i < 256 ; i++ )
		{
			redDst[i]	= redSrc[i];
			greenDst[i]	= greenSrc[i];
			blueDst[i]	= blueSrc[i];
			grayDst[i]	= graySrc[i];
		}
	}
	
	ret = CNMS_NO_ERR;
EXIT:

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjSetHistogram ]\n" );
#endif

	return ret;
}


/***   Get ColorGroup Number from Advanced ColorMode   ***/
CNMSInt32 CnmsColAdjGetColorGroupNum(
						CNMSVoid									)
{
	CNMSByte		*str		= CNMSNULL;
	CNMSInt32		ret			= CNMS_ERR;
	GtkWidget		*ComboBox	= CNMSNULL;

	if( main_window == CNMSNULL ){
		DBGMSG( "[CnmsColAdjGetColorGroupNum] Pointer[main_window] is NULL.\n" );
		goto	EXIT;
	}
	if( ( ComboBox = lookup_widget( main_window, "advance_color_combo" ) ) == CNMSNULL ){
		DBGMSG( "[CnmsColAdjGetColorGroupNum] Pointer[ComboBox] is NULL.\n" );
		goto	EXIT;
	}
	if( ( str = gtk_combo_box_get_active_text( GTK_COMBO_BOX( ComboBox ) ) ) == CNMSNULL ){
		DBGMSG( "[CnmsColAdjGetColorGroupNum] Pointer[str] is NULL.\n" );
		goto	EXIT;
	}

	/* Select Group from ColorMode str */
	     if( CnmsStrCompare( str, CnmsChgValToStr( CNMS_A_COLOR_MODE_COLOR          , CNMS_OBJ_A_COLOR_MODE ) ) == CNMS_NO_ERR ){
		ret = CNMS_CA_COLORGROUP_COLOR;
	}
	else if( CnmsStrCompare( str, CnmsChgValToStr( CNMS_A_COLOR_MODE_COLOR_DOCUMENTS, CNMS_OBJ_A_COLOR_MODE ) ) == CNMS_NO_ERR ){
		ret = CNMS_CA_COLORGROUP_COLOR;
	}
	else if( CnmsStrCompare( str, CnmsChgValToStr( CNMS_A_COLOR_MODE_GRAY           , CNMS_OBJ_A_COLOR_MODE ) ) == CNMS_NO_ERR ){
		ret = CNMS_CA_COLORGROUP_GRAY;
	}
	else if( CnmsStrCompare( str, CnmsChgValToStr( CNMS_A_COLOR_MODE_MONO           , CNMS_OBJ_A_COLOR_MODE ) ) == CNMS_NO_ERR ){
		ret = CNMS_CA_COLORGROUP_MONO;
	}
	else{
		ret = CNMS_CA_COLORGROUP_ERROR;
	}

EXIT:

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjGetColorGroupNum ]\n" );
#endif

	return ret;
}


/***   ComboBox Item All Clear   ***/
CNMSVoid CnmsColAdjComboBoxItemAllClear(
						GtkComboBox			*combobox				)
{
	CNMSInt32		i		= 0;
	
	if( combobox == CNMSNULL ){
		DBGMSG( "[CnmsColAdjComboBoxItemAllClear] Arg Pointer[combobox] is NULL.\n" );
		return;
	}
	
	ca_combobox_clear_flag = CNMS_TRUE;
	ComGtk_ClearCombo( combobox );
	ca_combobox_clear_flag = CNMS_FALSE;

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjComboBoxItemAllClear ]\n" );
#endif

	return;
}


/***   Set GdkGC Color from RGB Value   ***/
CNMSVoid CnmsColAdjGdkGCSetColor(
						GdkGC				*gc,
						CNMSUInt16			r,
						CNMSUInt16			g,
						CNMSUInt16			b						)
{
	GdkColor		color;

	if( gc == CNMSNULL ){
		DBGMSG( "[CnmsColAdjGdkGCSetColor] Arg Pointer[gc] is NULL.\n" );
		return;
	}

	color.red = r;
	color.green = g;
	color.blue = b;
	gdk_color_alloc(gdk_colormap_get_system(), &color);
	gdk_gc_set_foreground(gc, &color);

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjGdkGCSetColor ]\n" );
#endif

	return;
}


/***   Calculate ToneCurve from Brigtness/Contrast Value   ***/
CNMSVoid CnmsColAdjCurveCalcBC(
						CNMSMAKETONEPARAM	*base_param				)
{
	CNMSMAKETONEPARAM	param;
	CNMSRGB32			usertone[256];
	CNMSRGB32			resulttone[256];
	CNMSInt32			i = 0;
	
	if( base_param == CNMSNULL ){
		DBGMSG( "[CnmsColAdjCurveCalcBC] Arg Pointer is NULL. base_param[%p]\n", base_param );
		return;
	}

	CnmsColAdjParamInit( &param, usertone, resulttone );

	param.brightness.r = base_param->brightness.r;
	param.brightness.g = base_param->brightness.g;
	param.brightness.b = base_param->brightness.b;

	param.contrast.r = base_param->contrast.r;
	param.contrast.g = base_param->contrast.g;
	param.contrast.b = base_param->contrast.b;
	
	lpCnmsImgApi->p_CnmsImgMakeToneCurve( &param, usertone, resulttone );
	
	for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ){
		bc_red.curve[i]   = resulttone[i].r;
		bc_green.curve[i] = resulttone[i].g;
		bc_blue.curve[i]  = resulttone[i].b;	
	}
	
	CnmsColAdjParamInit( &param, usertone, resulttone );
	
	param.brightness.m = base_param->brightness.m;
	param.contrast.m   = base_param->contrast.m;

	lpCnmsImgApi->p_CnmsImgMakeToneCurve( &param, usertone, resulttone );

	for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ){
		bc_master.curve[i] = resulttone[i].r;
		bc_gray.curve[i]  = resulttone[i].r;
	}

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjCurveCalcBC ]\n" );
#endif

	return;
}


/***   Calculate ToneCurve from Shadow/Mid/Highlight Value   ***/
CNMSVoid CnmsColAdjCurveCalcHG(
						CNMSMAKETONEPARAM	*base_param				)
{
	CNMSMAKETONEPARAM	param;
	CNMSRGB32			usertone[256];
	CNMSRGB32			resulttone[256];
	CNMSInt32			i = 0;
	
	if( base_param == CNMSNULL ){
		DBGMSG( "[CnmsColAdjCurveCalcHG] Arg Pointer is NULL. base_param[%p]\n", base_param );
		return;
	}

	CnmsColAdjParamInit( &param, usertone, resulttone );
	
	param.blackpoint.r = base_param->blackpoint.r;
	param.blackpoint.g = base_param->blackpoint.g;
	param.blackpoint.b = base_param->blackpoint.b;

	param.midpoint.r = base_param->midpoint.r;
	param.midpoint.g = base_param->midpoint.g;
	param.midpoint.b = base_param->midpoint.b;

	param.whitepoint.r = base_param->whitepoint.r;
	param.whitepoint.g = base_param->whitepoint.g;
	param.whitepoint.b = base_param->whitepoint.b;

	param.graypoint.r = base_param->graypoint.r;
	param.graypoint.g = base_param->graypoint.g;
	param.graypoint.b = base_param->graypoint.b;

	lpCnmsImgApi->p_CnmsImgMakeToneCurve( &param, usertone, resulttone );
	
	for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ){
		hg_red.curve[i]   = resulttone[i].r;
		hg_green.curve[i] = resulttone[i].g;
		hg_blue.curve[i]  = resulttone[i].b;	
	}
	
	CnmsColAdjParamInit( &param, usertone, resulttone );
	
	param.blackpoint.m = base_param->blackpoint.m;
	param.midpoint.m = base_param->midpoint.m;
	param.whitepoint.m = base_param->whitepoint.m;

	lpCnmsImgApi->p_CnmsImgMakeToneCurve( &param, usertone, resulttone );

	for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ){
		hg_master.curve[i] = resulttone[i].r;
		hg_gray.curve[i]  = resulttone[i].r;
	}

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjCurveCalcHG ]\n" );
#endif

	return;
}


/***   Calculate ToneCurve from All ColorAdjust Parameter Value   ***/
CNMSVoid CnmsColAdjCurveCalcFR(
						CNMSMAKETONEPARAM	*param,
						CNMSRGB32			*usertone,
						CNMSRGB32			*resulttone				)
{
	CNMSInt32			i = 0;

	if( ( param == CNMSNULL ) && ( usertone == CNMSNULL ) && ( resulttone == CNMSNULL ) ){
		DBGMSG( "[CnmsColAdjCurveCalcFR] Arg Pointer is NULL. param[%p], usertone[%p], resulttone[%p]\n", param, usertone, resulttone );
		return;
	}

	lpCnmsImgApi->p_CnmsImgMakeToneCurve( param, usertone, resulttone );

	for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ){
		fr_red.curve[i]   = resulttone[i].r;
		fr_green.curve[i] = resulttone[i].g;
		fr_blue.curve[i]  = resulttone[i].b;	
		fr_gray.curve[i]  = resulttone[i].r;
	}

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjCurveCalcFR ]\n" );
#endif

	return;
}


/***   Initialize All ColorAdjust Parameter Value   ***/
CNMSVoid CnmsColAdjParamInit(
						CNMSMAKETONEPARAM	*param,
						CNMSRGB32			*usertone,
						CNMSRGB32			*resulttone				)
{
	CNMSInt32			i 						= 0;
	CNMSUInt8			tc_curve_out[256];
	CNMSUInt16			tc_curve_out16[65536];

	/* All arg pointer NULL is Static Prame Init	*/
	if( ( param == CNMSNULL ) && ( usertone == CNMSNULL ) && ( resulttone == CNMSNULL ) ){
		param		= &ca_param_all;
		usertone	= ca_usertone;
		resulttone	= ca_resulttone;

		lpCnmsImgApi->p_CnmsImgSplineCalc16( tc_curve_out16, CNMS_TC_NO_ADJUST );
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN * 256 ; i++ ){
			ca_usertone16[i].m   = ca_usertone16[i].r   = ca_usertone16[i].g   = ca_usertone16[i].b   = tc_curve_out16[i];
		}
		lpCnmsImgApi->p_CnmsImgMakeToneCurve16( param, ca_usertone16, ca_resulttone16 );
	}
	/* Someone arg pointer NULL is Error			*/
	else if( !( ( param != CNMSNULL ) && ( usertone != CNMSNULL ) && ( resulttone != CNMSNULL ) ) ){
		DBGMSG( "[CnmsColAdjParamInit] Arg Pointer is NULL. param[%p], usertone[%p], resulttone[%p]\n", param, usertone, resulttone );
		return;
	}

	param->blackpoint.m = param->blackpoint.r = param->blackpoint.g = param->blackpoint.b =   0;
	param->midpoint.m   = param->midpoint.r   = param->midpoint.g   = param->midpoint.b   = 128;
	param->whitepoint.m = param->whitepoint.r = param->whitepoint.g = param->whitepoint.b = 255;
	param->graypoint.m  = param->graypoint.r  = param->graypoint.g  = param->graypoint.b  = 128;

	param->brightness.m = param->brightness.r = param->brightness.g = param->brightness.b = 128;
	param->contrast.m   = param->contrast.r   = param->contrast.g   = param->contrast.b   = 128;
	param->balance.m    = param->balance.r    = param->balance.g    = param->balance.b    = 128;
	param->gamma.m      = param->gamma.r      = param->gamma.g      = param->gamma.b      = 1.0;

	lpCnmsImgApi->p_CnmsImgSplineCalc( tc_curve_out, CNMS_TC_NO_ADJUST );

	for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ){
		usertone[i].m   = usertone[i].r   = usertone[i].g   = usertone[i].b   = tc_curve_out[i];
	}
	CnmsColAdjCurveCalcFR( param, usertone, resulttone );

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjParamInit ]\n" );
#endif

	return;
}


/***   Preview Image Set From Change ColorAdjust Parameter   ***/
CNMSVoid CnmsColAdjPreviewImageSet(
						CNMSVoid									)
{
#ifdef	__CNMS_DEBUG_CALLED_SCANFLOW__
	DBGMSG("------------------------------------------- Preview_ChangeStatus -----------------------------------------\n");
#endif
	Preview_ChangeStatus( CNMSSCPROC_ACTION_PARAM );
	
#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjPreviewImageSet ]\n" );
#endif

	return;
}


/***   Set UserTone Called From ScanMain   ***/
CNMSVoid CnmsColAdjSetUserTone(
						CNMSUInt16			*R,
						CNMSUInt16			*G,
						CNMSUInt16			*B						)
{
	CNMSInt32	i	= 0;
	
	if( ( ca_usertone16 == CNMSNULL ) || ( ca_resulttone16 == CNMSNULL ) ){
		DBGMSG( "[CnmsColAdjSetUserTone] Pointer is NULL. ca_usertone16[%p], ca_resulttone16[%p]\n", ca_usertone16, ca_resulttone16 );
		return;
	}
	if( ( R == CNMSNULL ) || ( G == CNMSNULL ) || ( B == CNMSNULL ) ){
		DBGMSG( "[CnmsColAdjSetUserTone] Arg Pointer is NULL. R[%p], G[%p], B[%p]\n",R,G,B );
		return;
	}
	
	lpCnmsImgApi->p_CnmsImgMakeToneCurve16( &ca_param_all, ca_usertone16, ca_resulttone16 );

	for( i = 0 ; i < 65536 ; i++ ){
		R[ i ] = ( CNMSUInt16 )ca_resulttone16[ i ].r;
		G[ i ] = ( CNMSUInt16 )ca_resulttone16[ i ].g;
		B[ i ] = ( CNMSUInt16 )ca_resulttone16[ i ].b;
	}
	
#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjSetUserTone ]\n" );
#endif

	return;
}


/***   Set Threshold Called From ScanMain   ***/
CNMSInt32 CnmsColAdjSetThreshold(
						CNMSVoid									)
{
	CNMSInt32	ret = CNMS_ERR;
	
	if( ( mc_mono.threshold < 0 ) || ( mc_mono.threshold > 255 ) ){
		DBGMSG( "[CnmsColAdjSetThreshold] Value is invalid. mc_mono.threshold[%d]\n", mc_mono.threshold );
		ret = CNMS_CA_MC_DEFAULT;
	}
	else{
		ret = mc_mono.threshold;
	}

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjSetThreshold ]\n" );
#endif

	return	ret;
}


/***   Set Default Threshold Called From ScanMain   ***/
CNMSInt32 CnmsColAdjSetThresholdDefault(
						CNMSVoid									)
{

#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjSetThresholdDefault ]\n" );
#endif

	return	CNMS_CA_MC_DEFAULT;
}


/***   Call By Only scanmain.c   */
CNMSVoid CnmsShowDialogKeep(
						CNMSVoid									)
{
	if( dialog_histogram == CNMSNULL ){
		CnmsHGDialogShow( 0, CNMS_CA_DIALOG_KEEP );
	}
	if( dialog_monochrome == CNMSNULL ){
		CnmsMCDialogShow( 0, CNMS_CA_DIALOG_KEEP );
	}
	if( dialog_final_review == CNMSNULL ){
		CnmsFRDialogShow( 0, CNMS_CA_DIALOG_KEEP );
	}
	return;
}	
	

/***   For Call ScanFlow By Parameter Changed Timing   */
CNMSVoid CnmsCallScanFlowByParamChangeTiming(
						CNMSInt32			reset_flag				)
{
#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG("\t[CnmsCallScanFlowByParamChangeTiming]\n");
#endif

	if( reset_flag == CNMS_FALSE ){
		CnmsColAdjPreviewImageSet();
	}
	return;
}	
	

/***   Dialog Position Initialize   ***/
CNMSVoid CnmsColAdjSetDialogInitPosition(
						CNMSDialogPos		*pos,
						GtkWidget			*dialog					)
{
	CNMSInt			WindowWidth		= 0,
					WindowHeight	= 0,
					DialogWidth		= 0,
					DialogHeight	= 0,
					WindowX			= 0,
					WindowY			= 0;
	CNMSInt32		DialogNum		= 0;
	
	if( ( dialog == CNMSNULL ) || ( pos == CNMSNULL ) ){
		DBGMSG( "[CnmsColAdjSetDialogInitPosition] Arg Pointer is NULL. dialog[%p],pos[%p]\n",dialog,pos );
		return;
	}
	
	if( 	 dialog == dialog_bright_contrast )		DialogNum = 1;
	else if( dialog == dialog_histogram )			DialogNum = 2;
	else if( dialog == dialog_tonecurve )			DialogNum = 3;
	else if( dialog == dialog_final_review )		DialogNum = 4;
	else if( dialog == dialog_monochrome )			DialogNum = 1;
	else{
		DBGMSG( "[CnmsColAdjSetDialogInitPosition] Arg Pointer is invalid. dialog[%p]\n",dialog );
		return;
	}		
	
	/* Get Main window Size & Position	*/
	gtk_window_get_position( GTK_WINDOW( main_window ), &WindowX, &WindowY );
	gtk_window_get_size( GTK_WINDOW( main_window ), &WindowWidth, &WindowHeight );
	
	gtk_window_get_size( GTK_WINDOW( dialog ), &DialogWidth, &DialogHeight );
	pos->x = WindowX + WindowWidth - DialogWidth - ( CNMS_CA_HEIGHT_TITLE_BAR * DialogNum );
	pos->y = WindowY + ( CNMS_CA_HEIGHT_TITLE_BAR * DialogNum );

	return;
}


/***   Normalize Histogram Value To Graph Height Max From Raw Value   ***/
static CNMSVoid CnmsColAdjCurveValueNormalize(
						CNMSUInt32			*curve,
						CNMSInt32			max_len,
						CNMSInt32			max_value				)
{
	CNMSInt32		max		= 0,
					i		= 0;

	if( curve == CNMSNULL ){
		DBGMSG( "[CnmsColAdjCurveValueNormalize] Arg Pointer[curve] is NULL.\n" );
		return;
	}

	for( i = 0 ; i < max_len ; i++ ){
		if( max < abs(curve[i]) ){
			max = abs(curve[i]);
		}
	}
	
	for( i = 0 ; i < max_len ; i++ ){
		curve[i] = ( CNMSUInt32 )( ( CNMSDec32 )abs( curve[i] ) * ( CNMSDec32 )max_value / ( CNMSDec32 )max );	
	}
	
#ifdef __COLOR_ADJUST_DEBUG__
	DBGMSG( "\t[ CnmsColAdjCurveValueNormalize ]\n" );
#endif

	return;
}


/***   Avoid Show Value "-0"   ***/
CNMSVoid CnmsColAdjSetZeroIfMinusZeroForSpinbutton(
						GtkWidget			*spinbutton				)
{

	if( spinbutton == CNMSNULL ){
		DBGMSG( "[CnmsColAdjSetZeroIfMinusZeroForSpinbutton] Arg Widget Pointer[spinbutton] is NULL.\n" );
		return;
	}
	
	if( strncmp( gtk_entry_get_text( GTK_ENTRY( spinbutton ) ), "-0", 2 ) == 0 ){		/* Avoid Show Value "-0"	*/
		gtk_spin_button_set_value( GTK_SPIN_BUTTON( spinbutton), 1 );
		gtk_spin_button_set_value( GTK_SPIN_BUTTON( spinbutton), 0 );
	}

	return;
}


#endif	/* _COLORADJUST_C_ */

