/*
 *  ScanGear MP for Linux
 *  Copyright CANON INC. 2007
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * NOTE:
 *  - As a special exception, this program is permissible to link with the
 *    libraries released as the binary modules.
 *  - If you write modifications of your own for these programs, it is your
 *    choice whether to permit this exception to apply to your modifications.
 *    If you do not wish that, delete this exception.
*/


#ifndef	_COLORADJUST_H_
#define	_COLORADJUST_H_

#ifdef	_CNMS_CA_GLOBALS_
	#define	CNMS_CA_GLOBALS
#else
	#define	CNMS_CA_GLOBALS	extern
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <math.h>

#include "interface.h"
#include "support.h"
#include "scanflow.h"

#include "w1.h"
#include "cnmstype.h"

#include "coloradjuststr.h"
#include "cnmscoloradjust.h"

#define		CNMS_CA_DIALOG_NON_KEEP		(	0)		/* Use for XX_dialog_show()							*/
#define		CNMS_CA_DIALOG_KEEP			(	1)		/* Ditto											*/

#define		CNMS_CA_VECTOR_LEN			(  256)		/* Table length (Number of member)					*/
#define		CNMS_CA_STRING_LEN			(  128)		/* Standard string length							*/

#define		CNMS_CA_GRAPH_FRAME_LEFT	(	3)		/* Graph drawing area frame coordinate 	: Left		*/
#define		CNMS_CA_GRAPH_FRAME_TOP		(	3)		/* Ditto								: Top		*/
#define		CNMS_CA_GRAPH_FRAME_RIGHT	( 227)		/* Ditto								: Right		*/
#define		CNMS_CA_GRAPH_FRAME_BOTTOM	( 227)		/* Ditto								: Bottom	*/

#define		CNMS_CA_HEIGHT_TITLE_BAR	(  20)		/* Title Bar Height for Dialog show position		*/

#define		CNMS_CA_MC_DEFAULT			(  72)		/* Default threshold								*/

#define		CNMS_CA_CHANNEL_MASTER		(   0)		/* Channel ComboBox List Number	: Master			*/
#define		CNMS_CA_CHANNEL_RED			(   1)		/* Ditto						: Red				*/
#define		CNMS_CA_CHANNEL_GREEN		(   2)		/* Ditto						: Green				*/
#define		CNMS_CA_CHANNEL_BLUE		(   3)		/* Ditto						: Blue				*/
#define		CNMS_CA_CHANNEL_GRAY		(   0)		/* Ditto						: Gray				*/

#define		CNMS_CA_GC_BACKGROUND_R		(0xEFEF)	/* 239 GC color R for graph background				*/
#define		CNMS_CA_GC_BACKGROUND_G		(0xEBEB)	/* 235 GC color G for graph background				*/
#define		CNMS_CA_GC_BACKGROUND_B		(0xE7E7)	/* 231 GC color B for graph background				*/

#define		CNMS_CA_GC_FRAME_R			(0xB1B1)	/* 177 GC color R for graph frame					*/
#define		CNMS_CA_GC_FRAME_G			(0xA5A5)	/* 165 GC color G for graph frame					*/
#define		CNMS_CA_GC_FRAME_B			(0x9898)	/* 152 GC color B for graph frame					*/

#define		CNMS_CA_GC_VALUE_MAX		(0xFFFF)	/* 255 GC color value max							*/
#define		CNMS_CA_GC_VALUE_ZERO		(0x0000)	/*   0 GC color value zero							*/
#define		CNMS_CA_GC_VALUE_GRAY		(0x7777)	/* 119 GC color value gray							*/


/*******************************************/
/***        Define for ColorAdjust       ***/
/*******************************************/

	CNMS_CA_GLOBALS	CNMSMAKETONEPARAM	ca_param_all;
	CNMS_CA_GLOBALS	CNMSRGB32			ca_usertone[CNMS_CA_VECTOR_LEN];
	CNMS_CA_GLOBALS	CNMSRGB32			ca_resulttone[CNMS_CA_VECTOR_LEN];
	CNMS_CA_GLOBALS	CNMSRGB32			ca_usertone16[CNMS_CA_VECTOR_LEN * 256];
	CNMS_CA_GLOBALS	CNMSRGB32			ca_resulttone16[CNMS_CA_VECTOR_LEN * 256];

	CNMS_CA_GLOBALS CNMSInt32			ca_current_colorgroup_num;			/* Use for keep current colorgroup	*/

enum{	/* Color Group */
	CNMS_CA_COLORGROUP_ERROR = -1,	/* Error		*/
	CNMS_CA_COLORGROUP_COLOR,		/* Color		*/
	CNMS_CA_COLORGROUP_GRAY,		/* Grayscale	*/
	CNMS_CA_COLORGROUP_MONO,		/* Monochrome 	*/
	CNMS_CA_COLORGROUP_MAX,
};

enum{	/* Color number, Use for gc set color */
	CNMS_CA_COLOR_BLACK = 0,		/* Black		*/
	CNMS_CA_COLOR_RED,				/* Red			*/
	CNMS_CA_COLOR_GREEN,			/* Green		*/
	CNMS_CA_COLOR_BLUE,				/* Blue			*/
	CNMS_CA_COLOR_GRAY,				/* Gray			*/
};



/*******************************************/
/***    Define for Brightness/Contrast   ***/
/*******************************************/

typedef struct{
		CNMSDec32 			bright;								/* Brightness value					*/
		CNMSDec32 			contrast;							/* Contrast value					*/
		CNMSInt32 			curve[CNMS_CA_VECTOR_LEN];			/* Brightness & Contrast Table		*/
}CNMSBCValue;

	CNMS_CA_GLOBALS GtkWidget		*bc_scale_bright,			/* ScaleBar   for brightness		*/
	                                *bc_spinbutton_bright,		/* SpinBottun for brightness		*/
	                                *bc_scale_contrast,			/* ScaleBar   for contrast			*/
	                                *bc_spinbutton_contrast,	/* SpinBottun for contrast			*/
	                                *bc_combo_box,				/* Channel ComboBox					*/
	                                *bc_drawarea;				/* DrawingArea for Graph			*/
                                
	CNMS_CA_GLOBALS	CNMSBCValue		bc_master,
									bc_red,
									bc_green,
									bc_blue,
									bc_gray;


/*******************************************/
/***         Define for Histogram        ***/
/*******************************************/

typedef struct{
		CNMSDec32 			raw;
		CNMSInt32			disp;
		CNMSInt32			lower;
		CNMSInt32			upper;
		CNMSInt32			center;
}CNMSHGScaleBarPos;

typedef struct{
		CNMSUInt32	 		hist[CNMS_CA_VECTOR_LEN];			/* Histogram						*/
		CNMSInt32 			curve[CNMS_CA_VECTOR_LEN];			/* Shadow & Mid & Highlight Table	*/
		CNMSHGScaleBarPos	shadow;
		CNMSHGScaleBarPos	mid;
		CNMSHGScaleBarPos	highlight;
}CNMSHGValue;

	CNMS_CA_GLOBALS GtkWidget		*hg_combo_box,				/* Channel ComboBox					*/
	                                *hg_drawarea,				/* DrawingArea for Graph			*/
	                                *hg_entry_shadow,			/* TextEntry for Shadow				*/
	                                *hg_entry_mid,				/* TextEntry for Mid				*/
	                                *hg_entry_highlight,		/* TextEntry for Highlight			*/
	                                *hg_scalebar_drawarea,		/* DrawingArea for ScaleBar			*/
	                                *hg_label_str_1,			/* Label 1 for R/K					*/
	                                *hg_label_str_2,			/* Label 2 for G					*/
	                                *hg_label_str_3,			/* Label 3 for B					*/
	                                *hg_label_value_1,			/* Value 1 for R/K					*/
	                                *hg_label_value_2,			/* Value 2 for G					*/
	                                *hg_label_value_3,			/* Value 3 for B					*/
	                                *hg_togglebutton_gbal;		/* Gray Balance Toggle Button		*/

	CNMS_CA_GLOBALS	CNMSHGValue		hg_master,
									hg_red,
									hg_green,
									hg_blue,
									hg_gray;


/*******************************************/
/***         Define for ToneCurve        ***/
/*******************************************/

typedef struct{
		CNMSInt32			tc_num;								/* ToneCurve number					*/
		CNMSInt32 			curve[CNMS_CA_VECTOR_LEN];			/* ToneCurve Table					*/
}CNMSTCValue;

#define		CNMS_TC_NO_ADJUST	CNMS_SPLINE_NO_ADJUST			/*	0 	*/
#define		CNMS_TC_EXP_OVER	CNMS_SPLINE_EXP_OVER			/*  1 	*/
#define		CNMS_TC_EXP_UNDER	CNMS_SPLINE_EXP_UNDER			/*  2 	*/
#define		CNMS_TC_HIGH_CONT	CNMS_SPLINE_HIGH_CONT			/*  3 	*/

	CNMS_CA_GLOBALS GtkWidget		*tc_combobox_channel,		/* Channel ComboBox					*/
                                	*tc_drawarea,				/* DrawingArea for Graph			*/
                                	*tc_combobox_tone_select;	/* ToneCurve Select ComboBox		*/
                                
	CNMS_CA_GLOBALS	CNMSTCValue		tc_master,
									tc_red,
									tc_green,
									tc_blue,
									tc_gray;


/*******************************************/
/***        Define for FinalReview       ***/
/*******************************************/

typedef struct{
		CNMSUInt32 			hist[CNMS_CA_VECTOR_LEN];			/* Histogram for Final Review		*/
		CNMSInt32 			curve[CNMS_CA_VECTOR_LEN];			/* Final Review Table				*/
}CNMSFRValue;

	CNMS_CA_GLOBALS GtkWidget		*fr_combo_box,				/* Channel ComboBox					*/
	                                *fr_drawarea,				/* DrawingArea for Graph			*/
	                                *fr_label_str_a1,			/* Label 1 After  for R/K			*/
	                                *fr_label_str_b1,	        /* Label 1 Before for R/K			*/
	                                *fr_label_str_a2,	        /* Label 2 After  for G				*/
	                                *fr_label_str_b2,	        /* Label 2 Before for G				*/
	                                *fr_label_str_a3,	        /* Label 3 After  for B				*/
	                                *fr_label_str_b3,	        /* Label 3 Before for B				*/
	                                *fr_label_value_a1,			/* Value 1 After  for R/K			*/
	                                *fr_label_value_b1,			/* Value 1 Before for R/K			*/
	                                *fr_label_value_a2,			/* Value 2 After  for G				*/
	                                *fr_label_value_b2,			/* Value 2 Before for G				*/
	                                *fr_label_value_a3,			/* Value 3 After  for B				*/
	                                *fr_label_value_b3;			/* Value 3 Before for B				*/

	CNMS_CA_GLOBALS	CNMSFRValue		fr_master,
									fr_red,
									fr_green,
									fr_blue,
									fr_gray;
									

/*******************************************/
/***        Define for Monochrome        ***/
/*******************************************/

typedef struct{
		CNMSUInt32 			hist[CNMS_CA_VECTOR_LEN];			/* Histogram for monochrome			*/
		CNMSInt32 			curve[CNMS_CA_VECTOR_LEN];			/* Monochrome Table					*/
		CNMSInt32			threshold;							/* Threshold						*/
}CNMSMCValue;

	CNMS_CA_GLOBALS GtkWidget		*mc_drawarea,				/* DrawingArea for Graph			*/
	                                *mc_spinbutton,				/* SpinButton for Threshold			*/
	                                *mc_scale;					/* ScaleBar for Threshold			*/

	CNMS_CA_GLOBALS	CNMSMCValue		mc_mono;
	

/*******************************************/
/***              Function               ***/
/*******************************************/


CNMSInt32 CnmsColAdjDrawAreaBaseDraw(
						GtkWidget			*widget,
						GdkPixmap			*pixmap,
						GdkGC				*gc						);

CNMSInt32 CnmsColAdjDrawAreaCurveDraw(
						GtkWidget			*widget,
						GdkPixmap			*pixmap,
						GdkGC				*gc,
						CNMSInt32			curve[],
						CNMSInt32			color					);

CNMSInt32 CnmsColAdjDrawAreaHistogramDraw(
						GtkWidget			*widget,
						GdkPixmap			*pixmap,
						GdkGC				*gc,
						CNMSUInt32			histogram[],
						CNMSInt32			color					);

CNMSInt32 CnmsColAdjDialogShow(
						GtkWidget			*dialog,
						CNMSDialogPos		*pos					);

CNMSInt32 CnmsColAdjGetColorGroupNum(
						CNMSVoid									);

CNMSVoid CnmsColAdjComboBoxItemAllClear(
						GtkComboBox			*combobox				);

CNMSVoid CnmsColAdjGdkGCSetColor(
						GdkGC				*gc,
						CNMSUInt16			r,
						CNMSUInt16			g,
						CNMSUInt16			b						);

CNMSVoid CnmsColAdjCurveCalcBC(
						CNMSMAKETONEPARAM	*base_param				);

CNMSVoid CnmsColAdjCurveCalcHG(
						CNMSMAKETONEPARAM	*base_param				);

CNMSVoid CnmsColAdjCurveCalcFR(
						CNMSMAKETONEPARAM	*param,
						CNMSRGB32			*usertone,
						CNMSRGB32			*resulttone				);

CNMSVoid CnmsColAdjPreviewImageSet(
						CNMSVoid									);

CNMSVoid CnmsCallScanFlowByParamChangeTiming(
						CNMSInt32			reset_flag				);

CNMSVoid CnmsColAdjSetDialogInitPosition(
						CNMSDialogPos		*pos,
						GtkWidget			*dialog					);

CNMSVoid CnmsColAdjSetZeroIfMinusZeroForSpinbutton(
						GtkWidget			*spinbutton				);

#endif	/* _COLORADJUST_H_ */
