/*
 *  ScanGear MP for Linux
 *  Copyright CANON INC. 2007
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * NOTE:
 *  - As a special exception, this program is permissible to link with the
 *    libraries released as the binary modules.
 *  - If you write modifications of your own for these programs, it is your
 *    choice whether to permit this exception to apply to your modifications.
 *    If you do not wish that, delete this exception.
*/

#ifndef	_COLORADJUST_BC_C_
#define	_COLORADJUST_BC_C_

#include "coloradjust_bc.h"

#define		BC_DEF_BRIGHT		(0)		/* Default Value for Brightness	*/
#define		BC_DEF_CONTRAST		(0)		/* Default Value for Contrast	*/

	static	CNMSInt32		bc_current_channel			= CNMS_CA_CHANNEL_MASTER;	/* For Keep Current Channel 				*/
	static	CNMSBCValue		*bc_current					= CNMSNULL;					/* Selecting Channel Value					*/
	static	GdkPixmap		*bc_pixmap_drawing_area		= CNMSNULL;					/* Pixmap For Graph Drawing Area 			*/
	static	GdkGC			*bc_current_gc_drawing_area	= CNMSNULL;					/* Graphic Context For Graph Drawing Area	*/
	static	CNMSInt32		bc_reset_flag				= CNMS_FALSE;				/* Flag	for Reset			 				*/


static CNMSVoid CnmsBCSetChannelData(
						CNMSBCValue			*current				);

static CNMSVoid CnmsBCToneCurveDraw(
						GtkWidget			*widget,
						CNMSInt32			color,
						CNMSInt32			channel					);


/***   Brightness/Contrast Dialog Show   ***/
CNMSVoid CnmsBCDialogShow(
						CNMSInt32			color,
						CNMSInt32			keep					)
{
	bc_reset_flag = CNMS_TRUE;

	/* Dialog Position Initialize */
	if( ( bc_pos.x == CNMS_ERR ) && ( bc_pos.y == CNMS_ERR ) ){
		CnmsColAdjSetDialogInitPosition( &bc_pos, dialog_bright_contrast );
	}

	/* Channel ComboBox Item Change depending on ColorGroup */
	switch( color )
	{
		case CNMS_CA_COLORGROUP_COLOR:
				CnmsColAdjComboBoxItemAllClear( GTK_COMBO_BOX( bc_combo_box ) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( bc_combo_box ), gettext(ca_ch_litem_master) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( bc_combo_box ), gettext(ca_ch_litem_red) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( bc_combo_box ), gettext(ca_ch_litem_green) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( bc_combo_box ), gettext(ca_ch_litem_blue) );
				ca_current_colorgroup_num = CNMS_CA_COLORGROUP_COLOR;
				break;
		case CNMS_CA_COLORGROUP_GRAY:
				CnmsColAdjComboBoxItemAllClear( GTK_COMBO_BOX( bc_combo_box ) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( bc_combo_box ), gettext(ca_ch_litem_gray) );
				ca_current_colorgroup_num = CNMS_CA_COLORGROUP_GRAY;
				break;
		case CNMS_CA_COLORGROUP_MONO:
				if( GTK_WIDGET_VISIBLE( dialog_bright_contrast ) != 0 )
					CnmsColAdjDialogHide( dialog_bright_contrast, &bc_pos );
				return;
		default:
				DBGMSG( "[CnmsBCDialogShow] Arg Value is invalid. color[%d]\n",color );
				return;
	}

	/* Show/UnShow Change */
	if( keep == CNMS_CA_DIALOG_NON_KEEP )
	{
		if( GTK_WIDGET_VISIBLE( dialog_bright_contrast ) == 0 ){
			CnmsColAdjDialogShow( dialog_bright_contrast, &bc_pos );
			gtk_combo_box_set_active( GTK_COMBO_BOX( bc_combo_box ), bc_current_channel );
		}
		else{
			CnmsColAdjDialogHide( dialog_bright_contrast, &bc_pos );
		}
	}
	else{
		if( GTK_WIDGET_VISIBLE( dialog_bright_contrast ) != 0 ){
			gtk_combo_box_set_active( GTK_COMBO_BOX( bc_combo_box ), 0 );
		}
	}

	bc_reset_flag = CNMS_FALSE;

	return;
}


/***   Data Initialize   ***/
CNMSVoid CnmsBCInit(
						CNMSVoid									)
{
	CNMSInt32	i = 0;

	/* Initialize Brightness Value	*/
	bc_master.bright = bc_red.bright = bc_green.bright = bc_blue.bright = bc_gray.bright = BC_DEF_BRIGHT;
	ca_param_all.brightness.m = ca_param_all.brightness.r = ca_param_all.brightness.g = ca_param_all.brightness.b = 128;
	
	/* Initialize Contrast Value	*/
	bc_master.contrast = bc_red.contrast = bc_green.contrast = bc_blue.contrast = bc_gray.contrast = BC_DEF_CONTRAST;
	ca_param_all.contrast.m = ca_param_all.contrast.r = ca_param_all.contrast.g = ca_param_all.contrast.b = 128;

	/* Initialize ToneCueve Value	*/
	for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ){
		bc_master.curve[i] = bc_red.curve[i] = bc_green.curve[i] = bc_blue.curve[i] = bc_gray.curve[i] = i;
	}
	
	return;
}


/***   Reset   ***/
CNMSVoid CnmsBCReset(
						CNMSVoid									)
{
	bc_reset_flag = CNMS_TRUE;

	/* Reset Displayed Brightness/Contrast	*/
	gtk_spin_button_set_value( GTK_SPIN_BUTTON( bc_spinbutton_bright), BC_DEF_BRIGHT );
	gtk_range_set_value( GTK_RANGE( bc_scale_bright ), BC_DEF_BRIGHT );
	gtk_spin_button_set_value( GTK_SPIN_BUTTON( bc_spinbutton_contrast), BC_DEF_CONTRAST );
	gtk_range_set_value( GTK_RANGE( bc_scale_contrast ), BC_DEF_CONTRAST );

	/* Data Initialize						*/
	CnmsBCInit();

	/* Channel Set ComboBox Top				*/
	bc_current_channel = 0;
	if( GTK_WIDGET_VISIBLE( dialog_bright_contrast ) != 0 ){
		gtk_combo_box_set_active( GTK_COMBO_BOX( bc_combo_box ), bc_current_channel );
	}

	/* Redraw FinalReview Data depending on Reset Brightness/Contrast */
	if( dialog_final_review != NULL ){
		if( GTK_WIDGET_VISIBLE( dialog_final_review ) != 0 ){
			CnmsColAdjCurveCalcFR( &ca_param_all, ca_usertone, ca_resulttone );
			CnmsFRToneCurveDrawFromOtherParamChanged();
		}
	}

	bc_reset_flag = CNMS_FALSE;
	
	return;
}


/***   Channel ComboBox Changed   ***/
CNMSVoid CnmsBCChannelComboBoxChanged(
						GtkComboBox			*combobox				)
{
	CNMSInt8	*channel	= CNMSNULL;

	/* If flag is True, Called by Combobox Clear func. The case, No action the process  */
	if( ca_combobox_clear_flag == CNMS_TRUE ){
		return;
	}

	if( combobox == CNMSNULL ){
		DBGMSG( "[CnmsBCChannelComboBoxChanged] Arg Pointer[combobox] is NULL.\n" );
		return;
	}
		
	/* Get Current Channel as String */
	if( ( channel = gtk_combo_box_get_active_text( combobox ) ) == CNMSNULL ){
		DBGMSG( "[CnmsBCChannelComboBoxChanged] Pointer[channel] is NULL.\n" );
		return;
	}
	
	if	   ( CnmsStrCompare( channel, gettext( ca_ch_litem_master ) ) == CNMS_NO_ERR ){
		bc_current_channel = CNMS_CA_CHANNEL_MASTER;									/* Convert Current Channel to Number from String	*/
		CnmsBCSetChannelData( &bc_master );												/* Set Data & ToneCurve Current Channel				*/
	}
	else if( CnmsStrCompare( channel, gettext( ca_ch_litem_red ) )    == CNMS_NO_ERR ){
		bc_current_channel = CNMS_CA_CHANNEL_RED;
		CnmsBCSetChannelData( &bc_red );
	}
	else if( CnmsStrCompare( channel, gettext( ca_ch_litem_green ) )  == CNMS_NO_ERR ){
		bc_current_channel = CNMS_CA_CHANNEL_GREEN;
		CnmsBCSetChannelData( &bc_green );
	}
	else if( CnmsStrCompare( channel, gettext( ca_ch_litem_blue ) )   == CNMS_NO_ERR ){
		bc_current_channel = CNMS_CA_CHANNEL_BLUE;
		CnmsBCSetChannelData( &bc_blue );
	}
	else if( CnmsStrCompare( channel, gettext( ca_ch_litem_gray ) )   == CNMS_NO_ERR ){
		bc_current_channel = CNMS_CA_CHANNEL_GRAY;
		CnmsBCSetChannelData( &bc_gray );
	}
	else{
		DBGMSG( "[CnmsBCChannelComboBoxChanged] Channel string[%s] is invalid.\n",channel );
	}
		
	return;
}


/***   Brightness Value Changed with ScaleBar   ***/
CNMSVoid CnmsBCScaleBrightValueChanged(
						GtkRange			*range					)
{

	if( ( range == CNMSNULL ) || ( bc_current == CNMSNULL ) ){
		DBGMSG( "[CnmsBCScaleBrightValueChanged] Pointer is NULL. range[%p], bc_current[%p]\n", range, bc_current );
		return;
	}

	/* Get ScaleBar Value & Set Value to SpinButton	*/
	bc_current->bright = gtk_range_get_value( range );
	gtk_spin_button_set_value( GTK_SPIN_BUTTON( bc_spinbutton_bright), bc_current->bright );
	CnmsColAdjSetZeroIfMinusZeroForSpinbutton( bc_spinbutton_bright );

	/* Keep Parameter depending on Current Channel	*/
	if( bc_current == &bc_master ){
		ca_param_all.brightness.m = bc_current->bright + 128;		/* -127 ~ +127 to 0 ~ +255	*/
	}
	else if( bc_current == &bc_red ){
		ca_param_all.brightness.r = bc_current->bright + 128;
	}
	else if( bc_current == &bc_green ){
		ca_param_all.brightness.g = bc_current->bright + 128;
	}
	else if( bc_current == &bc_blue ){
		ca_param_all.brightness.b = bc_current->bright + 128;
	}
	else if( bc_current == &bc_gray ){								/* Case Gray, Keep to Master Area							*/
		ca_param_all.brightness.m = bc_current->bright + 128;		/* No Need, Because Keep Master & Gray Value at One Time	*/
	}
	else{
		DBGMSG( "[CnmsBCScaleBrightValueChanged] Pointer [bc_current] is invalid.\n" );
		return;
	}

	/* Get ToneCurve Value & Draw ToneCurve			*/
	CnmsColAdjCurveCalcBC( &ca_param_all );
	CnmsBCToneCurveDraw( bc_drawarea, ca_current_colorgroup_num, bc_current_channel );

	/* Get FR ToneCurve Value & Draw FR ToneCurve	*/
	if( dialog_final_review != CNMSNULL ){
		if( GTK_WIDGET_VISIBLE( dialog_final_review ) != 0 ){
			CnmsColAdjCurveCalcFR( &ca_param_all, ca_usertone, ca_resulttone );
			CnmsFRToneCurveDrawFromOtherParamChanged();
		}
	}

	/* Case Flag OFF, Redraw Preview Data by Parameter Changed */
	CnmsCallScanFlowByParamChangeTiming( bc_reset_flag );

	return;
}


/***   Brightness Value Changed with SpinButton   ***/
CNMSVoid CnmsBCSpinButtonBrightValueChanged(
						GtkSpinButton		*spinbutton				)
{
	if( spinbutton == CNMSNULL ){
		DBGMSG( "[CnmsBCSpinButtonBrightValueChanged] Arg pointer [spinbutton] is NULL.\n" );
		return;
	}

	/* Get SpinButton Value & Set Value to ScaleBar	*/
	bc_current->bright = gtk_spin_button_get_value( spinbutton );
	gtk_range_set_value( GTK_RANGE( bc_scale_bright ), bc_current->bright );
	
	return;
}


/***   Contrast Value Changed with ScaleBar   ***/
CNMSVoid CnmsBCScaleContrastValueChanged(
						GtkRange			*range					)
{
	if( ( range == CNMSNULL ) || ( bc_current == CNMSNULL ) ){
		DBGMSG( "[CnmsBCScaleBrightValueChanged] Pointer is NULL. range[%p], bc_current[%p]\n", range, bc_current );
		return;
	}

	/* Get ScaleBar Value & Set Value to SpinButton	*/
	bc_current->contrast = gtk_range_get_value( range );
	gtk_spin_button_set_value( GTK_SPIN_BUTTON( bc_spinbutton_contrast), bc_current->contrast );
	CnmsColAdjSetZeroIfMinusZeroForSpinbutton( bc_spinbutton_contrast );

	/* Keep Parameter depending on Current Channel	*/
	if( bc_current == &bc_master ){
		ca_param_all.contrast.m = bc_current->contrast + 128;		/* -127 ~ +127 to 0 ~ +255	*/
	}
	else if( bc_current == &bc_red ){
		ca_param_all.contrast.r = bc_current->contrast + 128;
	}
	else if( bc_current == &bc_green ){
		ca_param_all.contrast.g = bc_current->contrast + 128;
	}
	else if( bc_current == &bc_blue ){
		ca_param_all.contrast.b = bc_current->contrast + 128;
	}
	else if( bc_current == &bc_gray ){								/* Case Gray, Keep as Master instead of Gray				*/
		ca_param_all.contrast.m = bc_current->contrast + 128;   	/* No Need, Because Keep Master & Gray Value at One Time	*/
	}
	else{
		DBGMSG( "[CnmsBCScaleContrastValueChanged] Pointer [bc_current] is invalid.\n" );
		return;
	}

	/* Get ToneCurve Value & Draw ToneCurve			*/
	CnmsColAdjCurveCalcBC( &ca_param_all );
	CnmsBCToneCurveDraw( bc_drawarea, ca_current_colorgroup_num, bc_current_channel );

	/* Get FR ToneCurve Value & Draw FR ToneCurve	*/
	if( dialog_final_review != CNMSNULL ){
		if( GTK_WIDGET_VISIBLE( dialog_final_review ) != 0 ){
			CnmsColAdjCurveCalcFR( &ca_param_all, ca_usertone, ca_resulttone );
			CnmsFRToneCurveDrawFromOtherParamChanged();
		}
	}

	/* Case Flag OFF, Redraw Preview Data by Parameter Changed */
	CnmsCallScanFlowByParamChangeTiming( bc_reset_flag );

	return;
}


/***   Contrast Value Changed with SpinButton   ***/
CNMSVoid CnmsBCSpinButtonContrastValueChanged(
						GtkSpinButton		*spinbutton				)
{
	if( spinbutton == CNMSNULL ){
		DBGMSG( "[CnmsBCSpinButtonContrastValueChanged] Arg pointer [spinbutton] is NULL.\n" );
		return;
	}

	/* Get SpinButton Value & Set Value to ScaleBar	*/
	bc_current->contrast = gtk_spin_button_get_value( spinbutton );
	gtk_range_set_value( GTK_RANGE( bc_scale_contrast ), bc_current->contrast );
	
	return;
}


/***   Event for Drawing Graph   ***/
CNMSVoid CnmsBCDrawingAreaConfigureEvent(
						GtkWidget			*widget,
						GdkEventConfigure	*event					)
{
	if( ( widget == CNMSNULL ) || ( event == CNMSNULL ) ){
		DBGMSG( "[CnmsBCDrawingAreaConfigureEvent] Arg Pointer is NULL. widget[%p], event[%p]\n", widget, event );
		return;
	}

	/* Create Pixmap			*/
	if ( bc_pixmap_drawing_area != CNMSNULL ){
		gdk_pixmap_unref( bc_pixmap_drawing_area );
	}
	bc_pixmap_drawing_area = gdk_pixmap_new( widget->window, widget->allocation.width, widget->allocation.height, -1 );

	/* Create Graphic Context	*/
	if( bc_current_gc_drawing_area == CNMSNULL ){
		bc_current_gc_drawing_area = gdk_gc_new( widget->window );
	}

	/* Draw ToneCurve			*/
	CnmsBCToneCurveDraw( widget, ca_current_colorgroup_num, bc_current_channel );
	
	return;
}


/***   Drawing Pixmap Event for Hiden Pixmap Area   ***/
CNMSVoid CnmsBCDrawingAreaExposeEvent(
						GtkWidget			*widget,
						GdkEventExpose		*event					)
{
	if( ( widget == CNMSNULL ) || ( event == CNMSNULL ) ){
		DBGMSG( "[CnmsBCDrawingAreaExposeEvent] Arg Pointer is NULL. widget[%p], event[%p]\n", widget, event );
		return;
	}

	/* Drawing Pixmap	*/
	gdk_draw_pixmap( widget->window, widget->style->fg_gc[ GTK_WIDGET_STATE( widget ) ],
		bc_pixmap_drawing_area, event->area.x, event->area.y, event->area.x, event->area.y, event->area.width, event->area.height );
		
	return;
}


/***   Set ToneCurve & Data depending on Channel   ***/
static CNMSVoid CnmsBCSetChannelData(
						CNMSBCValue			*current				)
{
	if( current == CNMSNULL ){
		DBGMSG( "[CnmsBCSetChannelData] Arg Pointer is NULL. current[%p]\n", current );
		return;
	}

	/* Set ToneCurve	*/
	bc_current = current;
	CnmsBCToneCurveDraw( bc_drawarea, ca_current_colorgroup_num, bc_current_channel );

	/* Set Data			*/
	gtk_spin_button_set_value( GTK_SPIN_BUTTON( bc_spinbutton_bright), bc_current->bright );
	gtk_range_set_value( GTK_RANGE( bc_scale_bright ), bc_current->bright );
	gtk_spin_button_set_value( GTK_SPIN_BUTTON( bc_spinbutton_contrast), bc_current->contrast );
	gtk_range_set_value( GTK_RANGE( bc_scale_contrast ), bc_current->contrast );

	return;
}


/***   Drawing ToneCurve depending on Channel   ***/
static CNMSVoid CnmsBCToneCurveDraw(
						GtkWidget			*widget,
						CNMSInt32			color,
						CNMSInt32			channel					)
{
	if( ( widget == CNMSNULL ) || ( bc_pixmap_drawing_area == CNMSNULL ) || ( bc_current_gc_drawing_area == CNMSNULL ) ){
		DBGMSG( "[CnmsBCToneCurveDraw] Arg Pointer is NULL. widget[%p], pixmap[%p], gc[%p]\n", widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area );
		return;
	}

	/* Drawing Graph Frame		*/
	CnmsColAdjDrawAreaBaseDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area );
	
	/* Set Graph ToneCurve		*/
	switch( color )
	{
		case CNMS_CA_COLORGROUP_COLOR:
			switch( channel )
			{
				case CNMS_CA_CHANNEL_MASTER:
					CnmsColAdjDrawAreaCurveDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area, bc_master.curve, CNMS_CA_COLOR_BLACK );
					break;
				
				case CNMS_CA_CHANNEL_RED:
					CnmsColAdjDrawAreaCurveDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area, bc_green.curve, CNMS_CA_COLOR_GREEN );
					CnmsColAdjDrawAreaCurveDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area, bc_blue.curve, CNMS_CA_COLOR_BLUE );
					CnmsColAdjDrawAreaCurveDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area, bc_red.curve, CNMS_CA_COLOR_RED );
					break;
				
				case CNMS_CA_CHANNEL_GREEN:
					CnmsColAdjDrawAreaCurveDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area, bc_red.curve, CNMS_CA_COLOR_RED );
					CnmsColAdjDrawAreaCurveDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area, bc_blue.curve, CNMS_CA_COLOR_BLUE );
					CnmsColAdjDrawAreaCurveDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area, bc_green.curve, CNMS_CA_COLOR_GREEN );
					break;
				
				case CNMS_CA_CHANNEL_BLUE:
					CnmsColAdjDrawAreaCurveDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area, bc_red.curve, CNMS_CA_COLOR_RED );
					CnmsColAdjDrawAreaCurveDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area, bc_green.curve, CNMS_CA_COLOR_GREEN );
					CnmsColAdjDrawAreaCurveDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area, bc_blue.curve, CNMS_CA_COLOR_BLUE );
					break;
				
				default:
					DBGMSG( "[CnmsBCToneCurveDraw] Arg value is invalid. channel[%d]\n", channel );
					break;
			}
			break;
			
		case CNMS_CA_COLORGROUP_GRAY:
			CnmsColAdjDrawAreaCurveDraw( widget, bc_pixmap_drawing_area, bc_current_gc_drawing_area, bc_gray.curve, CNMS_CA_COLOR_GRAY );
			break;
			
		default:
			DBGMSG( "[CnmsBCToneCurveDraw] Arg value is invalid. color[%d]\n", color );
			break;
	}

	/* Drawing Pixmap			*/
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		bc_pixmap_drawing_area, 0, 0, 0, 0, widget->allocation.width, widget->allocation.height);
		
	return;
}

#endif	/* _COLORADJUST_BC_C_ */

