/*
 *  ScanGear MP for Linux
 *  Copyright CANON INC. 2007
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * NOTE:
 *  - As a special exception, this program is permissible to link with the
 *    libraries released as the binary modules.
 *  - If you write modifications of your own for these programs, it is your
 *    choice whether to permit this exception to apply to your modifications.
 *    If you do not wish that, delete this exception.
*/

#ifndef	_COLORADJUST_MC_C_
#define	_COLORADJUST_MC_C_

#include "coloradjust_mc.h"

#define		MC_GC_UNDER_THRESHOLD		(0xD0D0)									/* 208 GC color under Threshold Background	*/

	static	GdkPixmap		*mc_pixmap_drawing_area		= CNMSNULL;					/* Pixmap For Graph Drawing Area 			*/
	static	GdkGC			*mc_current_gc_drawing_area	= CNMSNULL;					/* Graphic Context For Graph Drawing Area	*/
	static	CNMSInt32		mc_reset_flag				= CNMS_FALSE;				/* Flag	for Reset			 				*/

static CNMSInt32 CnmsWidthConvertFrom255(
						CNMSInt32			co_input,
						CNMSInt32			co_max					);

static CNMSVoid CnmsMCDrawThreshold(
						GtkWidget			*widget,
						CNMSUInt32			histogram[],
						CNMSInt32			value					);


/***   Monochrome Dialog Show   ***/
CNMSVoid CnmsMCDialogShow(
						CNMSInt32			color,
						CNMSInt32			keep					)
{
	mc_reset_flag = CNMS_TRUE;
	
	/* Dialog Position Initialize */
	if( ( mc_pos.x == CNMS_ERR ) && ( mc_pos.y == CNMS_ERR ) ){
		CnmsColAdjSetDialogInitPosition( &mc_pos, dialog_monochrome );
	}

	/* Channel ComboBox Item Change depending on ColorGroup */
	switch( color )
	{
		case CNMS_CA_COLORGROUP_COLOR:
		case CNMS_CA_COLORGROUP_GRAY:
				if( GTK_WIDGET_VISIBLE( dialog_monochrome ) != 0 )
					CnmsColAdjDialogHide( dialog_monochrome, &mc_pos );
				return;
		case CNMS_CA_COLORGROUP_MONO:
				break;
		default:
				DBGMSG( "[CnmsMCDialogShow] Arg Value is invalid. color[%d]\n",color );
				return;
	}
		
	/* Show/UnShow Change */
	if( keep == CNMS_CA_DIALOG_NON_KEEP )
	{
		if( GTK_WIDGET_VISIBLE( dialog_monochrome ) == 0 ){
			CnmsColAdjDialogShow( dialog_monochrome, &mc_pos );
			CnmsMCToneCurveDrawFromPreview();
		}
		else{
			CnmsColAdjDialogHide( dialog_monochrome, &mc_pos );
		}
	}

	mc_reset_flag = CNMS_FALSE;

	return;
}


/***   Data Initialize   ***/
CNMSVoid CnmsMCInit(
						CNMSBool			histFlag				)
{
	CNMSInt32	i = 0;

	/* Set Default Threshold Value	*/
	mc_mono.threshold = CNMS_CA_MC_DEFAULT;

	/* Initialize HistoGram Value	*/
	if( histFlag == TRUE ){
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ){
			mc_mono.hist[i] = 0;
		}
		CnmsMCToneCurveDrawFromPreview();
	}
	
	return;
}


/***   Reset   ***/
CNMSVoid CnmsMCReset(
						CNMSBool			histFlag				)
{
	mc_reset_flag = CNMS_TRUE;

	/* Data Initialize						*/
	CnmsMCInit( histFlag );
	gtk_range_set_value( GTK_RANGE( mc_scale ), ( CNMSDec32 )mc_mono.threshold );

	mc_reset_flag = CNMS_FALSE;

	return;
}


/***   Threshold Value Changed with ScaleBar   ***/
CNMSVoid CnmsMCScaleThresholdValueChanged(
						GtkRange			*range					)
{
	if( range == CNMSNULL ){
		DBGMSG( "[CnmsMCScaleThresholdValueChanged] Pointer is NULL. range[%p]\n", range );
		return;
	}

	/* Set Value to SpinButton			*/
	gtk_spin_button_set_value( GTK_SPIN_BUTTON( mc_spinbutton ), gtk_range_get_value( range ) );
	CnmsColAdjSetZeroIfMinusZeroForSpinbutton( mc_spinbutton );

	/* Draw Threshold					*/
	CnmsMCDrawThreshold( mc_drawarea, mc_mono.hist, mc_mono.threshold );

	/* Draw Preview Image Set Threshold	*/
	CnmsCallScanFlowByParamChangeTiming( mc_reset_flag );
	
	return;
}


/***   Threshold Value Changed with SpinButton   ***/
CNMSVoid CnmsMCSpinButtonThresholdValueChanged(
						GtkSpinButton		*spinbutton				)
{
	if( spinbutton == CNMSNULL ){
		DBGMSG( "[CnmsMCSpinButtonThresholdValueChanged] Arg pointer [spinbutton] is NULL.\n" );
		return;
	}

	/* Set Value to ScaleBar	*/
	mc_mono.threshold = gtk_spin_button_get_value_as_int( spinbutton );
	gtk_range_set_value( GTK_RANGE( mc_scale ), ( CNMSDec32 )mc_mono.threshold );
	
	return;
}


/***   Event for Drawing Graph   ***/
CNMSVoid CnmsMCDrawingAreaConfigureEvent(
						GtkWidget			*widget,
						GdkEventConfigure	*event					)
{
	if( ( widget == CNMSNULL ) || ( event == CNMSNULL ) ){
		DBGMSG( "[CnmsMCDrawingAreaConfigureEvent] Arg Pointer is NULL. widget[%p], event[%p]\n", widget, event );
		return;
	}

	/* Create Pixmap			*/
	if ( mc_pixmap_drawing_area != CNMSNULL ){
		gdk_pixmap_unref( mc_pixmap_drawing_area );
	}
	mc_pixmap_drawing_area = gdk_pixmap_new( widget->window, widget->allocation.width, widget->allocation.height, -1 );

	/* Create Graphic Context	*/
	if( mc_current_gc_drawing_area == CNMSNULL )
		mc_current_gc_drawing_area = gdk_gc_new( widget->window );

	/* Draw Histogram			*/
	CnmsMCDrawThreshold( widget, mc_mono.hist, mc_mono.threshold );
	
	return;
}


/***   Drawing Pixmap Event for Hiden Pixmap Area   ***/
CNMSVoid CnmsMCDrawingAreaExposeEvent(
						GtkWidget			*widget,
						GdkEventExpose		*event					)
{
	if( ( widget == CNMSNULL ) || ( event == CNMSNULL ) ){
		DBGMSG( "[CnmsMCDrawingAreaExposeEvent] Arg Pointer is NULL. widget[%p], event[%p]\n", widget, event );
		return;
	}

	/* Drawing Pixmap	*/
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		mc_pixmap_drawing_area, event->area.x, event->area.y, event->area.x, event->area.y, event->area.width, event->area.height);
		
	return;
}


/***   Process for ReDraw Histogram, when Preview Crop Area Changed   ***/
CNMSVoid CnmsMCToneCurveDrawFromPreview(
						CNMSVoid									)
{
	if( dialog_monochrome != CNMSNULL ){
		if( GTK_WIDGET_VISIBLE( dialog_monochrome ) != 0 ){
			CnmsMCDrawThreshold( mc_drawarea, mc_mono.hist, mc_mono.threshold );
		}
	}
	
	return;
}


/***   Draw Threshold Bar   ***/
static CNMSVoid CnmsMCDrawThreshold(
						GtkWidget			*widget,
						CNMSUInt32			histogram[],
						CNMSInt32			value					)
{
	CNMSInt32	Left			= CNMS_CA_GRAPH_FRAME_LEFT,
				Top				= CNMS_CA_GRAPH_FRAME_TOP,
				Right			= CNMS_CA_GRAPH_FRAME_RIGHT,
				Bottom			= CNMS_CA_GRAPH_FRAME_BOTTOM,
				XPosRedBar		= CnmsWidthConvertFrom255( value, ( Right - Left - 1 ) - 0.5 ) + ( Left + 1 ),
				XPosUnderThre	= CnmsWidthConvertFrom255( value, ( Right - Left - 1 ) - 0.5 );
	GdkPixmap	*Pixmap			= mc_pixmap_drawing_area;
	GdkGC		*Gc				= mc_current_gc_drawing_area;
				
	if( ( widget == CNMSNULL ) || ( histogram == CNMSNULL ) || ( Pixmap == CNMSNULL ) || ( Gc == CNMSNULL ) ){
		DBGMSG( "[CnmsMCDrawThreshold] Pointer is NULL. widget[%p], histogram[%p], Pixmap[%p], Gc[%p]\n", widget, histogram, Pixmap, Gc );
		return;
	}

	/* Draw	Graph Frame									*/
	CnmsColAdjDrawAreaBaseDraw( widget, Pixmap, Gc );

	/* Draw Background for Under Threshold				*/
	CnmsColAdjGdkGCSetColor( Gc, MC_GC_UNDER_THRESHOLD, MC_GC_UNDER_THRESHOLD, MC_GC_UNDER_THRESHOLD );
	gdk_draw_rectangle (Pixmap, Gc, TRUE, ( Left + 1 ), ( Top + 1 ), XPosUnderThre, ( Bottom - Top - 1 ) );

	/* Process above Hide Base Frame, ReDraw Base Frame	*/ 
	CnmsColAdjGdkGCSetColor( Gc, CNMS_CA_GC_FRAME_R, CNMS_CA_GC_FRAME_G, CNMS_CA_GC_FRAME_B );
	gdk_draw_line( Pixmap, Gc, 	( ( Right - Left ) * 1 / 4 ) + Left, Top, ( ( Right - Left ) * 1 / 4 ) + Left, Bottom ); 
	gdk_draw_line( Pixmap, Gc, 	( ( Right - Left ) * 2 / 4 ) + Left, Top, ( ( Right - Left ) * 2 / 4 ) + Left, Bottom ); 
	gdk_draw_line( Pixmap, Gc, 	( ( Right - Left ) * 3 / 4 ) + Left, Top, ( ( Right - Left ) * 3 / 4 ) + Left, Bottom ); 
	gdk_draw_line( Pixmap, Gc,	Left, ( ( Bottom - Top ) * 1 / 4 ) + Top, Right,( ( Bottom - Top ) * 1 / 4 ) + Top ); 
	gdk_draw_line( Pixmap, Gc,	Left, ( ( Bottom - Top ) * 2 / 4 ) + Top, Right,( ( Bottom - Top ) * 2 / 4 ) + Top ); 
	gdk_draw_line( Pixmap, Gc,	Left, ( ( Bottom - Top ) * 3 / 4 ) + Top, Right,( ( Bottom - Top ) * 3 / 4 ) + Top ); 

	/* Draw	Histogram									*/
	CnmsColAdjDrawAreaHistogramDraw( widget, Pixmap, Gc, mc_mono.hist, CNMS_CA_COLOR_BLACK );

	/* Draw	Red Bar for Threshold						*/
	CnmsColAdjGdkGCSetColor( Gc, CNMS_CA_GC_VALUE_MAX, CNMS_CA_GC_VALUE_ZERO, CNMS_CA_GC_VALUE_ZERO );
	gdk_draw_line( Pixmap, Gc, XPosRedBar, ( Top + 1 ), XPosRedBar, ( Bottom - 1 ) ); 

	/* Draw to Pixmap									*/
	gdk_draw_pixmap( widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)], Pixmap, 0, 0, 0, 0, widget->allocation.width, widget->allocation.height );

	return;
}


/***   Coordinate Convert to Red Bar Width from 255   ***/
static CNMSInt32 CnmsWidthConvertFrom255(
						CNMSInt32			co_input,
						CNMSInt32			co_max					)
{
	return ( CNMSInt32 )( ( ( CNMSDec32 )co_input * ( CNMSDec32 )co_max / 255 ) + 0.5 );
}


#endif	/* _COLORADJUST_MC_C_ */


