/*
 *  ScanGear MP for Linux
 *  Copyright CANON INC. 2007
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * NOTE:
 *  - As a special exception, this program is permissible to link with the
 *    libraries released as the binary modules.
 *  - If you write modifications of your own for these programs, it is your
 *    choice whether to permit this exception to apply to your modifications.
 *    If you do not wish that, delete this exception.
*/

#ifndef	_COLORADJUST_TC_C_
#define	_COLORADJUST_TC_C_

#include "cnmsfunc.h"
#include "coloradjust_tc.h"

	static	CNMSInt32		tc_current_channel			= CNMS_CA_CHANNEL_MASTER;	/* For Keep Current Channel 				*/
	static	CNMSTCValue		*tc_current					= CNMSNULL;					/* Selecting Channel Value					*/
	static	GdkPixmap		*tc_pixmap_drawing_area		= CNMSNULL;					/* Pixmap For Graph Drawing Area 			*/
	static	GdkGC			*tc_current_gc_drawing_area	= CNMSNULL;					/* Graphic Context For Graph Drawing Area	*/
	static	CNMSInt32		tc_reset_flag				= CNMS_FALSE;				/* Flag	for Reset			 				*/


static CNMSVoid CnmsTCSetChannelData(
						CNMSTCValue			*current				);

static CNMSVoid CnmsTCToneCurveDraw(
						GtkWidget			*widget,
						CNMSInt32			color,
						CNMSInt32			channel					);


/***   ToneCurve Adjustment Dialog Show   ***/
CNMSVoid CnmsTCDialogShow(
						CNMSInt32			color,
						CNMSInt32			keep					)
{
	tc_reset_flag = CNMS_TRUE;
	
	/* Dialog Position Initialize */
	if( ( tc_pos.x == CNMS_ERR ) && ( tc_pos.y == CNMS_ERR ) ){
		CnmsColAdjSetDialogInitPosition( &tc_pos, dialog_tonecurve );
	}

	/* Channel ComboBox Item Change depending on ColorGroup */
	switch( color )
	{
		case CNMS_CA_COLORGROUP_COLOR:
				CnmsColAdjComboBoxItemAllClear( GTK_COMBO_BOX( tc_combobox_channel ) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( tc_combobox_channel ), gettext(ca_ch_litem_master) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( tc_combobox_channel ), gettext(ca_ch_litem_red) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( tc_combobox_channel ), gettext(ca_ch_litem_green) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( tc_combobox_channel ), gettext(ca_ch_litem_blue) );
				ca_current_colorgroup_num = CNMS_CA_COLORGROUP_COLOR;
				break;
		case CNMS_CA_COLORGROUP_GRAY:
				CnmsColAdjComboBoxItemAllClear( GTK_COMBO_BOX( tc_combobox_channel ) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( tc_combobox_channel ), gettext(ca_ch_litem_gray) );
				ca_current_colorgroup_num = CNMS_CA_COLORGROUP_GRAY;
				break;
		case CNMS_CA_COLORGROUP_MONO:
				if( GTK_WIDGET_VISIBLE( dialog_tonecurve ) != 0 )
					CnmsColAdjDialogHide( dialog_tonecurve, &tc_pos );
				return;
		default:
				DBGMSG( "[CnmsTCDialogShow] Arg Value is invalid. color[%d]\n",color );
				return;
	}
		
	/* Show/UnShow Change */
	if( keep == CNMS_CA_DIALOG_NON_KEEP )
	{
		if( GTK_WIDGET_VISIBLE( dialog_tonecurve ) == 0 ){
			CnmsColAdjDialogShow( dialog_tonecurve, &tc_pos );
			gtk_combo_box_set_active( GTK_COMBO_BOX( tc_combobox_channel ), tc_current_channel );
			gtk_combo_box_set_active( GTK_COMBO_BOX( tc_combobox_tone_select ), tc_current->tc_num );
		}
		else{
			CnmsColAdjDialogHide( dialog_tonecurve, &tc_pos );
		}
	}
	else{
		if( GTK_WIDGET_VISIBLE( dialog_tonecurve ) != 0 ){
			gtk_combo_box_set_active( GTK_COMBO_BOX( tc_combobox_channel ), 0 );
			gtk_combo_box_set_active( GTK_COMBO_BOX( tc_combobox_tone_select ), 0 );
		}
	}
	
	tc_reset_flag = CNMS_FALSE;

	return;
}


/***   Data Initialize   ***/
CNMSVoid CnmsTCInit(
						CNMSVoid									)
{
	CNMSInt32	i = 0;

	/* Initialize ToneCurve Mode	*/
	tc_master.tc_num = tc_red.tc_num = tc_green.tc_num = tc_blue.tc_num = tc_gray.tc_num = CNMS_TC_NO_ADJUST;

	/* Initialize ToneCueve Value	*/
	for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ){
		tc_master.curve[i] = tc_red.curve[i] = tc_green.curve[i] = tc_blue.curve[i] = tc_gray.curve[i] = i;
	}
	
	return;
}


/***   Reset   ***/
CNMSVoid CnmsTCReset(
						CNMSVoid									)
{
	CNMSInt32		i						= 0;
	CNMSUInt8		tc_curve_out[256];
	CNMSUInt16		tc_curve_out16[65536];

	tc_reset_flag = CNMS_TRUE;

	/* Data Initialize						*/
	CnmsTCInit();

	/* Channel Set ComboBox Top				*/
	tc_current_channel = 0;
	if( GTK_WIDGET_VISIBLE( dialog_tonecurve ) != 0 ){
		gtk_combo_box_set_active( GTK_COMBO_BOX( tc_combobox_channel ), tc_current_channel );
		gtk_combo_box_set_active( GTK_COMBO_BOX( tc_combobox_tone_select ), CNMS_TC_NO_ADJUST );
	}

	lpCnmsImgApi->p_CnmsImgSplineCalc( tc_curve_out, CNMS_TC_NO_ADJUST );
	lpCnmsImgApi->p_CnmsImgSplineCalc16( tc_curve_out16, CNMS_TC_NO_ADJUST );

	for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ){
		ca_usertone[i].m = ca_usertone[i].r = ca_usertone[i].g = ca_usertone[i].b = tc_curve_out[i];
	}
	for( i = 0 ; i < CNMS_CA_VECTOR_LEN * 256 ; i++ ){
		ca_usertone16[i].m = ca_usertone16[i].r = ca_usertone16[i].g = ca_usertone16[i].b = tc_curve_out16[i];
	}

	/* Redraw FinalReview Data depending on Reset ToneCurve Adjustment */
	if( dialog_final_review != CNMSNULL ){
		if( GTK_WIDGET_VISIBLE( dialog_final_review ) != 0 ){
			CnmsColAdjCurveCalcFR( &ca_param_all, ca_usertone, ca_resulttone );
			CnmsFRToneCurveDrawFromOtherParamChanged();
		}
	}

	tc_reset_flag = CNMS_FALSE;
	
	return;
}


/***   Channel ComboBox Changed   ***/
CNMSVoid CnmsTCChannelComboBoxChanged(
						GtkComboBox			*combobox				)
{
	CNMSInt8	*channel	= CNMSNULL;

	/* If flag is True, Called by Combobox Clear func. The case, No action the process  */
	if( ca_combobox_clear_flag == CNMS_TRUE ){
		return;
	}

	if( combobox == CNMSNULL ){
		DBGMSG( "[CnmsTCChannelComboBoxChanged] Arg Pointer[combobox] is NULL.\n" );
		return;
	}
		
	/* Get Current Channel as String */
	if( ( channel = gtk_combo_box_get_active_text( combobox ) ) == CNMSNULL ){
		DBGMSG( "[CnmsBCChannelComboBoxChanged] Pointer[channel] is NULL.\n" );
		return;
	}

	if		( CnmsStrCompare( channel, gettext( ca_ch_litem_master ) ) == CNMS_NO_ERR ){
		tc_current_channel = CNMS_CA_CHANNEL_MASTER;									/* Convert Current Channel to Number from String	*/
		CnmsTCSetChannelData( &tc_master );												/* Set Data & ToneCurve Current Channel				*/
	}
	else if	( CnmsStrCompare( channel, gettext( ca_ch_litem_red ) )    == CNMS_NO_ERR ){
		tc_current_channel = CNMS_CA_CHANNEL_RED;
		CnmsTCSetChannelData( &tc_red );
	}
	else if	( CnmsStrCompare( channel, gettext( ca_ch_litem_green ) )  == CNMS_NO_ERR ){
		tc_current_channel = CNMS_CA_CHANNEL_GREEN;
		CnmsTCSetChannelData( &tc_green );
	}
	else if	( CnmsStrCompare( channel, gettext( ca_ch_litem_blue ) )   == CNMS_NO_ERR ){
		tc_current_channel = CNMS_CA_CHANNEL_BLUE;
		CnmsTCSetChannelData( &tc_blue );
	}
	else if	( CnmsStrCompare( channel, gettext( ca_ch_litem_gray ) )   == CNMS_NO_ERR ){
		tc_current_channel = CNMS_CA_CHANNEL_GRAY;
		CnmsTCSetChannelData( &tc_gray );
	}
	else{
		DBGMSG( "[CnmsTCChannelComboBoxChanged] Channel string[%s] is invalid.\n",channel );
	}
		
	return;
}


/***   ToneCurve Adjustment ConboBox Changed   ***/
CNMSVoid CnmsTCToneCurveSelectComboBoxChanged(
						GtkComboBox			*combobox				)
{
	CNMSInt8		*tone					= CNMSNULL;
	CNMSUInt8		tc_curve_out[256];
	CNMSUInt16		tc_curve_out16[65536];
	CNMSInt32		i						= 0;

	if( combobox == CNMSNULL ){
		DBGMSG( "[CnmsTCToneCurveSelectComboBoxChanged] Arg Pointer is NULL. combobox[%p]\n",combobox );
		return;
	}		

	/* Get Selected ToneCurve String	*/
	if( ( tone = gtk_combo_box_get_active_text( combobox ) ) == CNMSNULL ){
		DBGMSG( "[CnmsTCToneCurveSelectComboBoxChanged] Pointer is NULL. tone[%p]\n",tone );
		return;
	}		
		
	/* Set Selected ToneCurve Parameter	*/
	CnmsSetMem( (CNMSLPSTR)tc_curve_out, 0, sizeof( tc_curve_out ) );
	CnmsSetMem( (CNMSLPSTR)tc_curve_out16, 0, sizeof( tc_curve_out16 ) );

	if		( CnmsStrCompare( tone, gettext( tc_litem_no_adjust ) )	== CNMS_NO_ERR ){		/* No Adjustment	*/
		tc_current->tc_num = CNMS_TC_NO_ADJUST;
		lpCnmsImgApi->p_CnmsImgSplineCalc( tc_curve_out, tc_current->tc_num );
		lpCnmsImgApi->p_CnmsImgSplineCalc16( tc_curve_out16, tc_current->tc_num );
	}
	else if	( CnmsStrCompare( tone, gettext( tc_litem_exp_over ) )	== CNMS_NO_ERR ){		/* Exposure over	*/
		tc_current->tc_num = CNMS_TC_EXP_OVER;
		lpCnmsImgApi->p_CnmsImgSplineCalc( tc_curve_out, tc_current->tc_num );
		lpCnmsImgApi->p_CnmsImgSplineCalc16( tc_curve_out16, tc_current->tc_num );
	}
	else if	( CnmsStrCompare( tone, gettext( tc_litem_exp_under ) )	== CNMS_NO_ERR ){		/* Exposure under	*/
		tc_current->tc_num = CNMS_TC_EXP_UNDER;
		lpCnmsImgApi->p_CnmsImgSplineCalc( tc_curve_out, tc_current->tc_num );
		lpCnmsImgApi->p_CnmsImgSplineCalc16( tc_curve_out16, tc_current->tc_num );
	}
	else if	( CnmsStrCompare( tone, gettext( tc_litem_high_cont ) )	== CNMS_NO_ERR ){		/* High contrast	*/
		tc_current->tc_num = CNMS_TC_HIGH_CONT;
		lpCnmsImgApi->p_CnmsImgSplineCalc( tc_curve_out, tc_current->tc_num );
		lpCnmsImgApi->p_CnmsImgSplineCalc16( tc_curve_out16, tc_current->tc_num );
	}
	else{
		DBGMSG( "[CnmsTCToneCurveSelectComboBoxChanged] String is invalid. tone[%s]\n",tone );
		return;
	}		
		
	/* Copy Spline Curve				*/
	for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ) {
		tc_current->curve[i] = ( CNMSUInt32 )(tc_curve_out[i]);
	}
	
	/* Keep Parameter depending on Current Channel	*/
	if( tc_current == &tc_master ){
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ) {
			ca_usertone[i].m = ( CNMSUInt32 )tc_curve_out[i];
		}
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN * 256 ; i++ ) {
			ca_usertone16[i].m = ( CNMSUInt32 )tc_curve_out16[i];
		}
	}
	else if( tc_current == &tc_red ){
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ) {
			ca_usertone[i].r = ( CNMSUInt32 )tc_curve_out[i];
		}
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN * 256 ; i++ ) {
			ca_usertone16[i].r = ( CNMSUInt32 )tc_curve_out16[i];
		}
	}
	else if( tc_current == &tc_green ){
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ) {
			ca_usertone[i].g = ( CNMSUInt32 )tc_curve_out[i];
		}
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN * 256 ; i++ ) {
			ca_usertone16[i].g = ( CNMSUInt32 )tc_curve_out16[i];
		}
	}
	else if( tc_current == &tc_blue ){
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ) {
			ca_usertone[i].b = ( CNMSUInt32 )tc_curve_out[i];
		}
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN * 256 ; i++ ) {
			ca_usertone16[i].b = ( CNMSUInt32 )tc_curve_out16[i];
		}
	}
	else if( tc_current == &tc_gray ){							/* Case Gray, Keep as Red instead of Gray	*/
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN ; i++ ) {
			ca_usertone[i].r = ( CNMSUInt32 )tc_curve_out[i];
		}
		for( i = 0 ; i < CNMS_CA_VECTOR_LEN * 256 ; i++ ) {
			ca_usertone16[i].m = ( CNMSUInt32 )tc_curve_out16[i];
		}
	}
	
	/* Get ToneCurve Value & Draw ToneCurve			*/
	CnmsTCToneCurveDraw( tc_drawarea, ca_current_colorgroup_num, tc_current_channel );

	/* Get FR ToneCurve Value & Draw FR ToneCurve	*/
	if( dialog_final_review != CNMSNULL ){
		if( GTK_WIDGET_VISIBLE( dialog_final_review ) != 0 ){
			CnmsColAdjCurveCalcFR( &ca_param_all, ca_usertone, ca_resulttone );
			CnmsFRToneCurveDrawFromOtherParamChanged();
		}
	}
	
	/* Case Flag OFF, Redraw Preview Data by Parameter Changed */
	CnmsCallScanFlowByParamChangeTiming( tc_reset_flag );

	return;
}


/***   Event for Drawing Graph   ***/
CNMSVoid CnmsTCDrawingAreaConfigureEvent(
						GtkWidget			*widget,
						GdkEventConfigure	*event					)
{
	if( ( widget == CNMSNULL ) || ( event == CNMSNULL ) ){
		DBGMSG( "[CnmsTCDrawingAreaConfigureEvent] Arg Pointer is NULL. widget[%p], event[%p]\n", widget, event );
		return;
	}

	/* Create Pixmap			*/
	if ( tc_pixmap_drawing_area != CNMSNULL ){
		gdk_pixmap_unref( tc_pixmap_drawing_area );
	}
	tc_pixmap_drawing_area = gdk_pixmap_new( widget->window, widget->allocation.width, widget->allocation.height, -1 );

	/* Create Graphic Context	*/
	if( tc_current_gc_drawing_area == CNMSNULL )
		tc_current_gc_drawing_area = gdk_gc_new( widget->window );

	/* Draw ToneCurve			*/
	CnmsTCToneCurveDraw( widget, ca_current_colorgroup_num, tc_current_channel );

	return;
}


/***   Drawing Pixmap Event for Hiden Pixmap Area   ***/
CNMSVoid CnmsTCDrawingAreaExposeEvent(
						GtkWidget			*widget,
						GdkEventExpose		*event					)
{
	if( ( widget == CNMSNULL ) || ( event == CNMSNULL ) ){
		DBGMSG( "[CnmsTCDrawingAreaExposeEvent] Arg Pointer is NULL. widget[%p], event[%p]\n", widget, event );
		return;
	}

	/* Drawing Pixmap	*/
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		tc_pixmap_drawing_area, event->area.x, event->area.y, event->area.x, event->area.y, event->area.width, event->area.height);

	return;
}


/***   Set ToneCurve & Data depending on Channel   ***/
static CNMSVoid CnmsTCSetChannelData(
						CNMSTCValue			*current				)
{
	if( current == CNMSNULL ){
		DBGMSG( "[CnmsTCSetChannelData] Arg Pointer is NULL. current[%p]\n", current );
		return;
	}

	/* Set ToneCurve	*/
	tc_current = current;
	CnmsTCToneCurveDraw( tc_drawarea, ca_current_colorgroup_num, tc_current_channel );

	/* Set Data			*/
	gtk_combo_box_set_active( GTK_COMBO_BOX( tc_combobox_tone_select ), tc_current->tc_num );

	return;
}


/***   Drawing ToneCurve depending on Channel   ***/
static CNMSVoid CnmsTCToneCurveDraw(
						GtkWidget			*widget,
						CNMSInt32			color,
						CNMSInt32			channel					)
{
	if( ( widget == CNMSNULL ) || ( tc_pixmap_drawing_area == CNMSNULL ) || ( tc_current_gc_drawing_area == CNMSNULL ) ){
		DBGMSG( "[CnmsTCToneCurveDraw] Arg Pointer is NULL. widget[%p], pixmap[%p], gc[%p]\n", widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area );
		return;
	}

	/* Drawing Graph Frame		*/
	CnmsColAdjDrawAreaBaseDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area );
	
	/* Set Graph ToneCurve		*/
	switch( color )
	{
		case CNMS_CA_COLORGROUP_COLOR:
			switch( channel )
			{
				case CNMS_CA_CHANNEL_MASTER:
					CnmsColAdjDrawAreaCurveDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area, tc_master.curve, CNMS_CA_COLOR_BLACK );
					break;
				
				case CNMS_CA_CHANNEL_RED:
					CnmsColAdjDrawAreaCurveDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area, tc_green.curve, CNMS_CA_COLOR_GREEN );
					CnmsColAdjDrawAreaCurveDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area, tc_blue.curve, CNMS_CA_COLOR_BLUE );
					CnmsColAdjDrawAreaCurveDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area, tc_red.curve, CNMS_CA_COLOR_RED );
					break;
				
				case CNMS_CA_CHANNEL_GREEN:
					CnmsColAdjDrawAreaCurveDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area, tc_red.curve, CNMS_CA_COLOR_RED );
					CnmsColAdjDrawAreaCurveDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area, tc_blue.curve, CNMS_CA_COLOR_BLUE );
					CnmsColAdjDrawAreaCurveDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area, tc_green.curve, CNMS_CA_COLOR_GREEN );
					break;
				
				case CNMS_CA_CHANNEL_BLUE:
					CnmsColAdjDrawAreaCurveDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area, tc_red.curve, CNMS_CA_COLOR_RED );
					CnmsColAdjDrawAreaCurveDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area, tc_green.curve, CNMS_CA_COLOR_GREEN );
					CnmsColAdjDrawAreaCurveDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area, tc_blue.curve, CNMS_CA_COLOR_BLUE );
					break;
				
				default:
					DBGMSG( "[CnmsTCToneCurveDraw] Arg value is invalid. channel[%d]\n", channel );
					break;
			}
			break;
			
		case CNMS_CA_COLORGROUP_GRAY:
			CnmsColAdjDrawAreaCurveDraw( widget, tc_pixmap_drawing_area, tc_current_gc_drawing_area, tc_gray.curve, CNMS_CA_COLOR_GRAY );
			break;
			
		default:
			DBGMSG( "[CnmsTCToneCurveDraw] Arg value is invalid. color[%d]\n", color );
			break;
	}

	/* Drawing Pixmap			*/
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		tc_pixmap_drawing_area, 0, 0, 0, 0, widget->allocation.width, widget->allocation.height);
	
	return;
}

#endif	/* _COLORADJUST_TC_C_ */

